// -*- c++ -*-
/*
 *  KDE support for MICO - A KApplication based class with a CORBA enhanced event loop.
 *  Copyright (C) 1997 Lars Doelle
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 mico@informatik.uni-frankfurt.de
 */

#ifndef __kdispatcher_h__
#define __kdispatcher_h__

#include <CORBA.h>
#include <qsocknot.h>
#include <kapp.h>

class QTimer;
class QTDispatcher;

/**
 *  The KFileNotifier class provides the notifying of file events.
 *
 *  @short Notifier for a file event.
 */
class KFileNotifier : public QObject
{
    Q_OBJECT

public:

    /**
     *  Constructor.
     */
    KFileNotifier ();

    /**
     *  Constructor.
     *
     *  @param _disp Pointer to the dispatcher object, which handles the notifier.
     *  @param _cb   Class that describes the function called to notify.
     *  @param _fd   Filedescriptor of the file.
     *  @param _evq  Type of the event, that will be sent.
     *  @param _evc  Event, that will be sent.
     */
    KFileNotifier( QTDispatcher* _disp, CORBA::DispatcherCallback* _cb, int _fd, QSocketNotifier::Type _evq, CORBA::Dispatcher::Event _evc );

    /**
     *  Destructor.
     */
    ~KFileNotifier ();

    QSocketNotifier* xid;
    CORBA::DispatcherCallback* cb;
    QTDispatcher* disp;
    CORBA::Dispatcher::Event ev;

private slots:

    void activated ( int _fd );
};

/**
 *  The KTimerNotifier class provides the notifying of timer events.
 *
 *  @short Notifier for a timer event.
 */
class KTimerNotifier : public QObject
{
    Q_OBJECT

public:

    /**
     *  Constructor.
     */
    KTimerNotifier ();

    /**
     *  Constructor.
     *
     *  @param _disp    Pointer to the dispatcher object, which handles the notifier.
     *  @param _cb      Class that describes the function called to notify.
     *  @param _timeout Timeout for the notifier.
     */
    KTimerNotifier( QTDispatcher* _disp, CORBA::DispatcherCallback* _cb, long _timeout );

    /**
     *  Destructor.
     */
    ~KTimerNotifier ();

    QTimer* xid;
    CORBA::DispatcherCallback* cb;
    QTDispatcher* disp;

private slots:

    void timeout();
};

/**
 *  The QTDispatcher class provides using Notifier-classes via CORBA.
 *
 *  Normally K applications that use CORBA should use KOMApplication, 
 *  but in situations where KOMApplication wont fit, the following 
 *  approach can be used:
 *  <pre>
 *  KApplication app(argc,argv);
 *  CORBA::ORB_var orb = CORBA::ORB_init( argc,argv);
 *  orb->dispatcher( new QTDispatcher( &app));
 *  // application init here
 *  orb->run(); // use this instead of app.exec()
 *  </pre>
 *  This is the "do-it-youself" method, so the programmer is responsible 
 *  for everything. The code to quit the application should do something 
 *  like:
 *  <pre>
 *  orb->dispatcher(0); // deactivates and deletes the QTDispatcher
 *  orb->shutdown();
 *  </pre>
 *  @short QTDispatcher class provides using Notifier-classes via CORBA.
 */
class QTDispatcher : public CORBA::Dispatcher
{

public:

    /**
     *  Constructor.
     *
     *  @param _app KOMApplication that will be the parent of the dispatcher.
     */
    QTDispatcher( KApplication* _app );

    /**
     *  Destructor.
     */
    virtual ~QTDispatcher();

    /**
     *  Defines a callback method for a READ event.
     *
     *  @param _cb Class that describes the function called to notify.
     *  @param _fd Filedescriptor of the watched file.
     */
    virtual void rd_event( CORBA::DispatcherCallback* _cb, CORBA::Long _fd );

    /**
     *  Defines a callback method for a WRITE event.
     *
     *  @param _cb Class that describes the function called to notify.
     *  @param _fd Filedescriptor of the watched file.
     */
    virtual void wr_event( CORBA::DispatcherCallback* _cb, CORBA::Long _fd );

    /**
     *  Defines a callback method for a EXCEPTION event.
     *
     *  @param _cb Class that describes the function called to notify.
     *  @param _fd Filedescriptor of the watched file.
     */
    virtual void ex_event( CORBA::DispatcherCallback* _cb, CORBA::Long _fd );

    /**
     *  Defines a callback method for a TIMEOUT event.
     *
     *  @param _cb      Class that describes the function called to notify.
     *  @param _timeout Timeout-time.
     */
    virtual void tm_event( CORBA::DispatcherCallback* _cb, CORBA::ULong _timeout );

    /**
     *  Removes a callback method from the event. Until then the method won't be
     *  called anymore if the event occurs.
     *
     *  @param _cb    Class that describes the function called to notify.
     *  @param _event Event.
     */
    virtual void remove( CORBA::DispatcherCallback* _cb, Event _event );

    /**
     *  Processes events by calling @ref KOMApplication::processNextEvent.
     *  The boolean argument indicates whether to process all events in an infinite loop
     *  or not. If FALSE only one event will be processed.
     *
     *  @param _infinite Infinite loop or not. Default-Value: TRUE.
     *
     *  @see KOMApplication::processNextEvent
     */
    virtual void run( CORBA::Boolean infinite = TRUE );

    /**
     *  Currently this method does nothing.
     *
     *  @param _disp Pointer to the dispatcher object, which handles the notifier.
     */
    virtual void move( CORBA::Dispatcher* _disp );

    /**
     *  Retrieves if there are file or timer events to be processed.
     *
     *  @return Return whether there are events to be processed or not.
     */
    virtual CORBA::Boolean idle() const;

    /**
     *  Removes a timer notifier.
     *
     *  @param _notifier KTimerNotifier, that will be removed.
     */
    void remove( KTimerNotifier* _notifier );

    KApplication* gfw;
    list<KFileNotifier*> fevents;
    list<KTimerNotifier*> tevents;
};


#endif /* __kdispatcher_h__ */
