/* This file is part of the KDE project
   Copyright (C) 1998, 1999 Torben Weis <weis@kde.org>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
 
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.
 
   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/     

#ifndef __kom__aggregate_h__
#define __kom__aggregate_h__

#include "kom.h"
#include "komComponent.h"

/**
 *  class KOMAggregateFactory
 *
 *  You have to implement these on your own!
 *  Needed Methods are:
 *
 *  @li ::KOM::Aggregate_ptr create( Component_ptr core );
 */
class KOMAggregateFactory : virtual public KOMComponentFactory,
			    virtual public KOM::AggregateFactory_skel
{

public:

};

/**
 *  A class derived from KOMAggregate can provide additional interfaces for a component.
 *
 *  Aggregates solve a problem with distributed objects, the problem of derivation. Since
 *  the implementation of an interface is completely enclosed there has to be another way
 *  to extend the functionality of an already existing object. By using aggregate components
 *  you can add new interfaces to an object, at run-time. This means you can extend the
 *  the functionality but you do not change the behavior of the object itself.
 *
 *  @short Provides additional interfaces for a component.
 *
 *  @see KOMPlugin
 */
class KOMAggregate : virtual public KOMComponent,
		     virtual public KOM::Aggregate_skel
{

public:

  /**
   *  Destructor.
   */
  virtual ~KOMAggregate();
  
  // C++
  /**
   *  This method cleans up memory when the object will be deleted.
   */
  virtual void cleanUp();

  // IDL
  /**
   *  Adds an aggregate.
   *
   *  @param _factory  AggregateFactoryPtr.
   *  @param _required InterfaceSeq.
   *  @param _provided InterfaceSeq.
   *  @param _activate Boolean.
   *
   *  @return Returns the ID of the added aggregate.
   */
  virtual KOM::ID addAggregate( KOM::AggregateFactory_ptr _factory,
				const KOM::InterfaceSeq& _required,
				const KOM::InterfaceSeq& _provided,
				CORBA::Boolean _activate);

  /**
   *  Removes an aggregate with the given ID.
   *
   *  @param _id ID of the aggregate that will be removed.
   */
  virtual void removeAggregate( KOM::ID _id );

  /**
   *  Adds an plugin.
   *
   *  @param _factory          PluginFactoryPtr.
   *  @param _required         InterfaceSeq.
   *  @param _required_plugins InterfaceSeq.
   *  @param _provided         InterfaceSeq.
   *  @param _activate         Boolean.
   *
   *  @return Returns the ID of the added plugin.
   */
  virtual KOM::ID addPlugin( KOM::PluginFactory_ptr _factory,
			     const KOM::InterfaceSeq& _required,
			     const KOM::InterfaceSeq& _required_plugins,
			     const KOM::InterfaceSeq& _provided,
			     CORBA::Boolean _activate);

  /**
   *  Removes a plugin with the given ID.
   *
   *  @param _id The ID of the Plugin, that shall be removed.
   */
  virtual void removePlugin( KOM::ID _id );
  
  /**
   *  Retrieves a pointer to an interface with a given name.
   *
   *  @param _name Name of the interface that shall be retrieved.
   *
   *  @return Pointer to an interface.
   */
  virtual CORBA::Object_ptr getInterface( const char* _name );

  /**
   *  Retrieves the supported interfaces.
   *
   *  @return Returns the supported interfaces.
   */
  virtual KOM::InterfaceSeq* interfaces();

  /**
   *  Checks if an interface with a given name is supported.
   *
   *  @param _name Name of the interface that wil be checked.
   *
   *  @return Returns whether the interface is supported or not.
   */
  virtual CORBA::Boolean supportsInterface( const char* _name );

  /**
   *  Retrieves a plugin-interface with a given name
   *
   *  @param _name Name of the plugin-interface that shall be retrieved.
   *
   *  @return Returns a ponter to the interface
   */
  virtual CORBA::Object_ptr getPluginInterface( const char* _name );

  /**
   *  Retrieves the supported plugin-interfaces.
   *
   *  @return Returns the supported plugin-interfaces.
   */
  virtual KOM::InterfaceSeq* pluginInterfaces();

  /**
   *  Checks if an plugin-interface with a given name is supported.
   *
   *  @param _name Name of the plugin-interface that wil be checked.
   *
   *  @return Returns whether the plugin-interface is supported or not.
   */
  virtual CORBA::Boolean supportsPluginInterface( const char* _name );

  /**
   *  Retrieves informations about all installed plugins.
   *
   *  @return Returns the data about the installed plugins.
   */
  virtual KOM::Component::PluginInfoSeq* describePlugins();

  /**
   *  Retrieves a pointer to a plugin with a given ID.
   *
   *  @param _id ID of the plugin that shall be retrieved.
   *
   *  @return Pointer to the returned plugin.
   */
  virtual KOM::Plugin_ptr getPlugin( KOM::ID _id );

  /**
   *  Retrieves a pointer to the parent component of the aggregate.
   *
   *  @return Pointer to a KOM::Component object.
   */
  virtual KOM::Component_ptr parent();
  
protected:

  /**
   *  Constructor.
   *
   *  @param _com Pointer to a KOM::Component object that will be the parent of the aggregate.
   */
  KOMAggregate( KOM::Component_ptr _com );
  
  KOM::Component_var m_vParent;

};

#endif

