// -*- c++ -*-
/*
 *  KDE support for MICO - A KApplication based class with a CORBA enhanced event loop.
 *  Copyright (C) 1997 Lars Doelle
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 mico@informatik.uni-frankfurt.de
 */

#ifndef __kom_app_h__
#define __kom_app_h__

/*
 * XXX ugly hack to make QApplication::quit_now accessible
 * to KOMApplication
 */
#define private protected
#include <qapplication.h>
#undef private

#include <qsocknot.h>
#include <kapp.h>

#undef None /* Damn X11! */
#include <kom.h>
#include <komComponent.h>

#define komapp     KOMApplication::komApplication()
#define komapp_orb KOMApplication::komApplication()->orb()
#define komapp_boa KOMApplication::komApplication()->boa()

//class QTimer;
class QTDispatcher;
class KOMApplicationIf;
class KOMShutdownManager;

/**
 *  This is the base class for all applications in KOM. It is derived from KApplication.
 */
class KOMApplication : public KApplication
{
    //friend QTDispatcher;
    friend KOMShutdownManager;

    Q_OBJECT

public:

    /**
     *  Constructor. Pass command-line arguments.
     *  
     *  Builds up a connection to the ORB and the BOA. Further it
     *  initializes a @ref QTDispatcher object.
     *  
     *  @param argc The number of arguments that are given on the
     *  command-line.
     *  @param argv Pointers to the arguments, that are given on the
     *  command-line.
     *  @param rAppName A string that represents the name of the application.
     */
    KOMApplication( int& argc, char** argv, const QString& rAppName = QString::null);

    /**
     *  Destructor
     */
    ~KOMApplication();

    /**
     *  This the KOM-equivalent to @ref QApplication::exec. It runs the ORB.
     *  This way the Application shows, that the implementation of all
     *  objects are ready.
     */
    virtual void exec();

    /**
     *  Retrieve the BOA object.
     *  
     *  @return A pointer to the BOA.
     */
    CORBA::BOA_ptr boa() { return pBoa; }

    /**
     *  Retrieve the ORB object.
     *  
     *  @return A pointer to the ORB.
     */
    CORBA::ORB_ptr orb() { return pOrb; }

    /**
     *  Retrieve wether the BOA is restoring objects or not.
     */
    virtual bool isRestoring() { return pBoa->restoring(); }

    /**
     *  Retrieve the current KOMApplication object.
     *  
     *  This is similar to the @ref KApplication::getKApplication. It allows
     *  access to the single global KOMApplication object, since more then one
     *  cannot be created in the same application, It saves the troubleof having
     *  to pass the pointer explicitly to every function that may require it.
     *  
     *  @return A pointer to the KOMApplication object.
     *
     *  @see KApplication
     */
    static KOMApplication* komApplication() { return m_pKOMApplication; }

protected:

    /**
     *  Starts a new instance of the app. This method is virtual and should be
     *  overwritten by a derived class. This method has to open all needed
     *  shells, views and documents.
     */
    virtual void start() { }

    /**
     *  Ends the existing instanc of the app. This method is virtual and should
     *  be overwritten by a derived class. This method cleans up all shells,
     *  views and documents.
     */
    virtual void end() { }

    /**
     *  Restores the app. This method is virtual and should be overwritten by
     *  a derived class. This method has to restore all needed shells, views
     *  and documents.
     */
    virtual void restore() { }

    /**
     *  Tell the BOA that the implementation of the server is ready.
     *
     *  @param _impl ImplementationDef.
     */
    virtual void impl_is_ready( CORBA::ImplementationDef_ptr _impl );

    /**
     *  Deactivate the implementation of the server in the BOA.
     *
     *  @param _impl ImplementationDef.
     */
    virtual void deactivate_impl( CORBA::ImplementationDef_ptr _impl );

    /**
     *  Tell the BOA that the implementation of a given object is ready.
     *
     *  @param _obj  Object.
     *  @param _impl ImplementationDef.
     */
    virtual void obj_is_ready( CORBA::Object_ptr _obj, CORBA::ImplementationDef_ptr _impl );

    /**
     *  Deactivate the implementation of an object in the BOA.
     *
     *  @param _obj  Object.
     */
    virtual void deactivate_obj( CORBA::Object_ptr _obj );

    /**
     *  Gets the next event and processes it.
     *
     *  @param _b Boolean.
     *
     *  @see QTDispatcher::run
     */
    virtual void processNextEvent( bool _b );

    /**
     *  Shuts the ORB down.
     *
     *  @param _wait Boolean;
     */
    virtual void shutdown( CORBA::Boolean _wait );
protected slots:
    /**
     *  Quit the Application.
     *  
     *  All views and documents will be deleted.
     */
    virtual void doquit();

private:

    enum State { Initial, ObjReady, ImplReady };
    CORBA::BOA_var pBoa;
    CORBA::ORB_var pOrb;
    State state;
    QTDispatcher* pQtd;
    static KOMApplication* m_pKOMApplication;
};

 /**
  *  class KOMApplicationIf
  */
class KOMApplicationIf : virtual public KOMComponent,
			 virtual public KOM::Application_skel
{

public:

    /**
     *  Constructor.
     */
    KOMApplicationIf();

    /**
     *  Constructor.
     *
     *  @param _refdata ReferenceData.
     */
    KOMApplicationIf( const CORBA::BOA::ReferenceData& _refdata );

    /**
     *  Constructor.
     *
     *  @param _obj Object.
     */
    KOMApplicationIf( CORBA::Object_ptr _obj );

    /**
     *  Destructor.
     */
    virtual ~KOMApplicationIf();
};

/**
 *  template class KOMBoot
 */
template<class X>
class KOMBoot : public CORBA::BOAObjectRestorer
{

public:

    /**
     *  Constructor.
     *
     *  @param _repoid RepoID.
     *  @param _tag    ObjectTag.
     */
    KOMBoot( const char* _repoid, const char* _tag )
    { m_strRepoId = _repoid; m_strTag = _tag; }

    /**
     *  restore.
     *
     *  @param _obj Object.
     */
    CORBA::Boolean restore( CORBA::Object_ptr _obj )
    {
      if ( m_strRepoId == _obj->_repoid() )
      {
	kdebug ( KDEBUG_INFO, 8003,  "restoring ... " );
	(void)new X( _obj );
	kdebug ( KDEBUG_INFO, 8003,  "restored ... " );
	return TRUE;
      }
      kdebug ( KDEBUG_INFO, 8003,  "cannot restore %i objects", _obj->_repoid() );
      return FALSE;
    }

    /**
     *  bind.
     *
     *  @param _repoid RepoID.
     *  @param _tag    ObjectTag.
     */
    CORBA::Boolean bind( const char* _repoid, const CORBA::ORB::ObjectTag& _tag )
    {
      if ( strcmp( _repoid, m_strRepoId ) == 0 )
      {
	CORBA::String_var str;
	str = komapp_orb->tag_to_string( _tag );
	if ( strcmp( str.in(), m_strTag ) != 0L )
	  return FALSE;

	kdebug ( KDEBUG_INFO, 8003, "creating '%s' ...", str.in() );
	(void)new X( _tag );
	return TRUE;
      }
      return FALSE;
    }

protected:

  QString m_strRepoId;
  QString m_strTag;
};

#endif // __kom_app_h__
