/* This file is part of the KDE project
   Copyright (C) 1998, 1999 Torben Weis <weis@kde.org>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
 
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.
 
   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/     

#ifndef __kom_base_h__
#define __kom_base_h__

#include "kom.h" 
#include "komVar.h"
#include "komShutdownManager.h"

#include <qvaluelist.h>
#include <qstring.h>
#include <qlist.h>
#include <qdict.h>

#include "kdebug.h"

#define SIGNAL_IMPL( name ) { ClientList *l = new QList<Client>; l->setAutoDelete( true ); m_dctSignals.insert( name, l ); }

#define EVENT_MAPPER( event, value ) { string __ev = event; const CORBA::Any& __v = value; 
#define MAPPING( event, type, func ) if ( __ev == event ) { type __t; if ( __v >>= __t ) return func( __t ); }
#define MAPPING_BOOLEAN( event, func ) if ( __ev == event ) { CORBA::Boolean __x; if ( __v >>= CORBA::Any::to_boolean( __x ) ) return func( __x ); }
#define MAPPING_OCTET( event, func ) if ( __ev == event ) { CORBA::Octet __x; if ( __v >>= CORBA::Any::to_octet( __x ) ) return func( __x ); }
#define MAPPING_CHAR( event, func ) if ( __ev == event ) { CORBA::Char __x; if ( __v >>= CORBA::Any::to_char( __x ) ) return func( __x ); }
#define MAPPING_WCHAR( event, func ) if ( __ev == event ) { CORBA::Wchar __x; if ( __v >>= CORBA::Any::to_wchar( __x ) ) return func( __x ); }
#define MAPPING_STRING( event, func ) if ( __ev == event ) { char *__x; if ( __v >>= CORBA::Any::to_string( __x, 0 ) ) return func( __x ); }
#define MAPPING_WSTRING( event, func ) if ( __ev == event ) { CORBA::WChar *__x; if ( __v >>= CORBA::Any::to_wstring( __x, 0 ) ) return func( __x ); }
#define MAPPING_OBJECT( event, func ) if ( __ev == event ) { CORBA::Object_ptr __x; if ( __v >>= CORBA::Any::to_object( __x ) ) return func( __x ); }
#define END_EVENT_MAPPER return false; }

#define EMIT_EVENT( dest, event, value ) { CORBA::Any __a; __a <<= value; dest->receive( event, __a ); }
#define EMIT_EVENT_BOOLEAN( dest, event, value ) { CORBA::Any __a; __a <<= CORBA::Any::from_boolean( value ); dest->receive( event, __a ); }
#define EMIT_EVENT_OCTET( dest, event, value ) { CORBA::Any __a; __a <<= CORBA::Any::from_octet( value ); dest->receive( event, __a ); }
#define EMIT_EVENT_CHAR( dest, event, value ) { CORBA::Any __a; __a <<= CORBA::Any::from_char( value ); dest->receive( event, __a ); }
#define EMIT_EVENT_WCHAR( dest, event, value ) { CORBA::Any __a; __a <<= CORBA::Any::from_wchar( value ); dest->receive( event, __a ); }
#define EMIT_EVENT_STRING( dest, event, value ) { CORBA::Any __a; __a <<= CORBA::Any::from_string( (char*)value, 0 ); dest->receive( event, __a ); }
#define EMIT_EVENT_WSTRING( dest, event, value ) { CORBA::Any __a; __a <<= CORBA::Any::from_wstring( (CORBA::WChar*)value, 0 ); dest->receive( event, __a ); }
#define EMIT_EVENT_OBJECT( dest, event, value ) { CORBA::Any __a; __a <<= CORBA::Any::from_object( value, "Object" ); dest->receive( event, __a ); }

int cmpIOR( CORBA::Object_ptr p1, CORBA::Object_ptr p2 );

/**
 *  The KOMBase class is the base class of KOM. 
 *
 *  The Base interface and its implementation provides the basic functionality for a
 *  standardized communication between CORBA objects using KOM. This includes
 *
 *  @li signals and slots, similar to the mechanism used in the QT library.
 *  @li event handling, including filtering
 *  @li relatives management
 *  @li KOM referencing
 *
 *  Events consist of two elements, the event name, being a string, and an event argument,
 *  being a CORBA::Any value and therefore freely choosable by the developer.
 *
 *  Events, sent to a specified obejct, can be imagined as being put through a pipe until they
 *  reach the destination object. The "pipe" is filled with installed event filters. There are
 *  three kinds of filters.
 *
 *  @li reading filters ( FM_READ )
 *  @li writing filters ( FM_WRITE )
 *  @li implementing filters ( FM_IMPLEMENT )
 *
 *  In the current implementation in KOM these filter modes do only specify the order how the
 *  event is processed. When an event is emitted it gets first filtered by all event filters
 *  with the filter mode FM_WRITE, the followed by FM_IMPLEMENT and finally by FM_READ. Event
 *  filters have two possibilities what they can do with the actual event: They can just read
 *  it or they can discard it, which means the event is discarded and will never receive its
 *  destination object.
 *
 *  @short The base class of KOM.
 */
class KOMBase : public virtual KOM::Base_skel
{  

public:

  /**
   *  The Client struct.
   */
  struct Client
  {
    /**
     *  Constructor.
     */
    Client( KOM::Base_ptr _obj, const char *_func ) 
    {
      obj = KOM::Base::_duplicate( _obj ); function = _func;
    }
    
    KOM::Base_var obj;
    QString function;
  };

  /**
   *  The Filter struct.
   */
  struct Filter
  {
    /**
     *  Constructor.
     */
    Filter( KOM::Base_ptr _o, const char *_func,
	    const KOM::EventTypeSeq& _ev, KOM::Base::FilterMode _mode )
    {
      obj = KOM::Base::_duplicate( _o );
      function = _func;
      events = _ev;
      mode = _mode;
    }
    
    KOM::EventTypeSeq events;
    KOM::Base_var obj;
    QString function;
    KOM::Base::FilterMode mode;
  };

  typedef QList<Client> ClientList;

public:

  /**
   *  Constructor.
   */
  KOMBase();

  /**
   *  Destructor.
   */
  virtual ~KOMBase();

  /**
   *  This is the KOM-equivalent to @ref QObject::connect.
   *  
   *  It connects a signal with method in a given KOM::Base object. One signal can be connected to
   *  several methods at a time. One method can be invoked by several signals. You can give
   *  parameters with a signal, all like QT.
   *  
   *  @param _signal   The name of the signal.
   *  @param _obj      A pointer to a KOM::Base object in that the given method will be executed.
   *  @param _function The name of the method, that will be executed, when the signal is emitted.
   *
   *  @see #disconnect, #connectNotify
   */
  virtual void connect( const char* _signal, KOM::Base_ptr _obj, const char* _function );

  /**
   *  This method will be called, when another object connects a signal to an objects slot.
   *
   *  @param _obj The object, that connects to the object.
   *
   *  @see #connect, #disconnectNotify
   */
  virtual void connectNotify( KOM::Base_ptr _obj );

  /**
   *  This is a KOM-equivalent to @ref QObject::disconnect. It disconnects a signal from method
   *  in a given KOM::Base object.
   *  
   *  @param _signal   The name of the signal.
   *  @param _obj      A pointer to a KOM::Base object, that the given method contains.
   *  @param _function The name of the method, that will be disconnected.
   *
   *  @see #connect, #disconnectNotify
   */
  virtual void disconnect( const char* _signal, KOM::Base_ptr _obj, const char* _function );

  /**
   *  This method will be called, when another object disconnects one of his signals from the
   *  one of the objects slots.
   *
   *  @param _obj The object that disconnects a signal from the object.
   *
   *  @see #connect, #connectNotify, #disconnect
   */
  virtual void disconnectNotify( KOM::Base_ptr _obj );

  /**
   *  Disconnect the given objects signals from our slots and the objects slots from our signals.
   *  In addition also all filters are deinstalled and all adoptions are annulated.
   *
   *  @param _obj A pointer to a KOM::Base object, that will be completely disconnected from
   *              the KOM::Base object.
   *
   *  @see #connect, #connectNotify, #disconnect, #disconnectAll, #disconnectObject, #disconnectSignalNotify, #disconnectFilterNotify
   */
  virtual void disconnectObject( KOM::Base_ptr _obj );

  /**
   *  Disconnect from all slots and signals. Do no longer filter events and remove all
   *  event filters from this object. In addition all adoptions are annullated.
   *
   *  @see #connect, #connectNotify, #disconnect, #disconnectNotify, #disconnectObject, #disconnectSignalNotify, #disconnectFilterNotify
   */
  virtual void disconnectAll();

  /**
   *  This method will be calles when an object disconnects a slot from one of the objects
   *  signals.
   *
   *  @param _obj      The object that disconnect his slot from the obejct.
   *  @param _signal   The signal that is disconnect.
   *  @param _function A method.
   */
  virtual void disconnectSignalNotify( KOM::Base_ptr _obj, const char* _signal, const char* _function );

  /**
   *  This method will be called when an object uninstall a filter to the object.
   *
   *  @param _obj The object, that uninstalled the filter.
   */
  virtual void disconnectFilterNotify( KOM::Base_ptr _obj );

  /**
   *  Retrieves all connections a KOM::Base object is currently connected.
   *
   *  @return Returns a list (sequence) of all connections.
   */
  KOM::Base::ConnectionSeq* describeConnections();

  /**
   *  Retrieves all installed event filters.
   *
   *  @return Returns a list (sequence) of all installed event filters.
   */
  KOM::Base::EventFilterSeq* describeEventFilters();

  /**
   *  Retrieves all relatives. Relatives are all adopted objects.
   *
   *  @return Returns a list (sequence) of all relatives.
   */
  KOM::Base::RelativesSeq* describeRelatives();
  
  /**
   *  Call this function to send an event to this object. The event will pass all installed
   *  event filters. If the event is not deleted by a filter, then @ref #event will be called
   *  afterwards to finally handle the event.
   *
   *  @param _name  Name of the event that will be processed.
   *  @param _value Value of the event that will be processed.
   *
   *  @see #receiveASync, #event
   */
  virtual void receive( const char* _name, const CORBA::Any& _value );

  /**
   *  Call this function to send an event to this object. The event will pass all installed
   *  event filters. If the event is not deleted by a filter, then @ref #event will be called
   *  afterwards to finally handle the event.
   *
   *  In the current implementation this method simply calls @ref #receive.
   *
   *  @param _name  Name of the event that will be processed.
   *  @param _value Value of the event that will be processed.
   *
   *  @see #receive, #event
   */
  virtual void receiveASync( const char* _name, const CORBA::Any& _value );

  /**
   *  Call this function to install an event filter. You can install more than one. The filters
   *  gets events the object sends to another object given as argument. The filter can delete an 
   *  event. When the event passes all filters it will passed to @ref #event.
   *  
   *  @param _obj      The object that will receive the events.
   *  @param _function A method.
   *  @param _events   Event type. Here you can specify which events will be put into the filter. 
   *  @param _mode     Filter mode. Can be FM_READ, FM_WRITE or FM_IMPLEMENT.
   *
   *  @see #event, #eventFilter, #uninstallFilter
   */
  virtual void installFilter( KOM::Base_ptr _obj, const char* _function, const KOM::EventTypeSeq& _events, KOM::Base::FilterMode _mode );

  /**
   *  Removes an installed event filter.
   *  
   *  @param _obj Object.
   *
   *  @see #event, #eventFilter, #installFilter
   */
  virtual void uninstallFilter( KOM::Base_ptr _obj );

  /**
   *  Processes event filtering.
   *
   *  @param _obj   Object that will receive the event. ?????
   *  @param _name  Name of the event.
   *  @param _value Value of the event.
   *
   *  @return Boolean.
   *
   *  @see #event
   */
  virtual CORBA::Boolean eventFilter( KOM::Base_ptr _obj, const char* _name, const CORBA::Any& _value );

  /**
   *  Increases the KOM reference counter. This should be used to "express" that you possess the 
   *  object. This gives you direct control over the lifecycle of the object by letting the
   *  reference counter act directly on the server object, in contrary to CORBA reference
   *  counting, where the reference counter only acts as on the stub object, in case of remote
   *  objects (this is different for local objects, where stub = server object).
   *
   *  @see #decRef, #refCount, #destroy
   */
  virtual void incRef();

  /**
   *  Decreases the KOM reference counter. This should be used to "express" that you don`t want
   *  to possess the object anymore. If the KOM reference counter fails below zero the object
   *  gets destroyed. This destruction is done by calling @ref #cleanUp.
   *
   *  @see #incRef, #refCount, #destroy, #cleanUp
   */
  virtual void decRef();

  /**
   *  Retrieves the value of the KOM reference counter of the object. This counter represents
   *  the number of KOM objects that are connected to the object. If the KOM reference counter
   *  fails below zero the object gets destroyed. This destruction is done by calling
   *  @ref #cleanUp.
   *
   *  @return The value of the KOM reference object.
   *
   *  @see #incRef, #decRef, #destroy
   */
  virtual CORBA::ULong refCount()
  { return m_ulRefCount; }

  /**
   *  This method checks the KOM reference counter. If it is zero, it destroys the current
   *  KOM::Base object by calling @ref #cleanUp.
   *
   *  @see #incRef, #decRef, #refCount, #cleanUp
   */
  virtual void destroy();
  
  /**
   *  Adopts the object given as paramter. You should use adopting whenever you want to hold
   *  a reference to an object and want to be informed when the object dies in order to free
   *  all your references to this object. But this should only be used if you are not the
   *  parent object, meaning you didn't reference the object directly via the KOM reference
   *  counter. When using adopting you should re-implement the @ref #leaveNotify (and perhaps
   *  @ref #adoptNotify) methods of your object (and don't forget to call the original
   *  KOMBase method!).
   *
   *  @param _ptr A pointer to the object that will be adopted.
   *
   *  @see #leave, #adoptNotify, #leaveNotify, #describeRelatives
   */
  virtual void adopt( KOM::Base_ptr _ptr );

  /**
   *  This method will be called, when another object adopts the object.
   *
   *  @param _ptr The object that adopts the object.
   *
   *  @see #adopt, #leave, #leaveNotify, #describeRelatives
   */
  virtual void adoptNotify( KOM::Base_ptr _ptr );

  /**
   *  Annulates an adoption of the object.
   *
   *  @param _ptr A pointer to the object that will be leaved.
   *
   *  @see #adopt, #adoptNotify, #leaveNotify, #describeRelatives
   */
  virtual void leave( KOM::Base_ptr _ptr );

  /**
   *  This method will be called, when another object annulates an adoption of the object.
   *
   *  @param _ptr A pointer the object, that leaves to object.
   *
   *  @see #leave, #adopt, #adoptNotify, #describeRelatives
   */
  virtual void leaveNotify( KOM::Base_ptr _ptr );

  /**
   *  Sets a KOM::ShutdownManager to watch this KOM::Base object. Normally this method
   *  will be called from @ref KOMShutdownManager::watchObject.
   *
   *  @param _manager A pointer to a KOM::ShutdownManager object, that watches this object.
   *
   *  @see #shutdownManager
   */
  void setShutdownManager( KOMShutdownManager *_manager ) { m_pShutdownManager = _manager; }

  /**
   *  This method retrieves a KOM::ShutdownManager object set by @ref setShutdownManager.
   *
   *  @return A pointer to a KOM::ShutdownManager object that watches the object.
   *
   *  @see #setShutdownManager
   */
  KOMShutdownManager *shutdownManager() { return m_pShutdownManager; }
  
protected:

  /**
   *  Processes all events that the object receives. An event will be received if it has passed
   *  all event filters. In the default implementation this method does nothing und returns
   *  false. You have to overload this method to process events.
   *
   *  @param _name  Name of the event that shall be processed.
   *  @param _value Value of the event that shall be processed.
   *
   *  @return Returns whether the event is processed or not.
   *
   *  @see #eventFilter
   */
  virtual bool event( const char* _type, const CORBA::Any& _value );

  // C++
  /**
   *  Disconnects all signals and slots from the current KOM::Base object to a given
   *  KOM::Base object.
   *  
   *  @param _obj         Object.
   *  @param _send_notify Bool.
   */
  virtual void disconnectObjectIntern( KOM::Base_ptr _obj, bool _send_notify );

  /**
   *  This method closes all connections to other objects and leaves all relatives. After
   *  this call is finished the object truly gets released.
   *
   *  You might want to re-implement the cleanUp()-method. In this case make sure that you
   *  don't forget two things:
   *
   *  @li only execute if the boolean variable m_bIsClean is false, otherwise simply return
   *  @li make sure you call the previous implementation!
   *
   *  @see #decRef
   */
  virtual void cleanUp();                                                                           

protected:

  QDict<ClientList> m_dctSignals;
  QList<KOM::Base_var> m_lstClients;
  QList<Filter> m_lstFilters;
  QValueList<KOM::Base_var> m_lstRelatives;
  bool m_bIsClean;
  CORBA::ULong m_ulRefCount;
                                                                                                    private:

  KOMShutdownManager* m_pShutdownManager;
};

void signal_call0( const char* _signal, QDict<KOMBase::ClientList>* _dict );

template <class P1>
void signal_call1( const char* _signal, QDict<KOMBase::ClientList>* _dict, P1 _p1 )
{
    KOMBase::ClientList *list = (*_dict)[ _signal ];
    if( list == 0L )
    {
      kdebug( KDEBUG_WARN, 8003, "Unknown signal %s", _signal );
      kdebug( KDEBUG_WARN, 8003, "Please insert SIGNAL_IMPL( \"%s\" ); in your "
	                         "constructor or choose a valid signal", _signal );
      assert( 0 );
    }
    
    QListIterator<KOMBase::Client> it( *list );
    for ( ; it.current(); ++it )
    {
        CORBA::Request_var _req = it.current()->obj->_request( it.current()->function.data() );
	_req->add_in_arg() <<= (P1)_p1;
	_req->result()->value()->type( CORBA::_tc_void );
	_req->invoke();
	// if( CORBA::Exception *_ex = _req->env()->exception() )
	// mico_throw( *_ex );
    }
}

template <class P1, class P2>
void signal_call2( const char* _signal, QDict<KOMBase::ClientList>* _dict, P1 _p1, P2 _p2 )
{
    KOMBase::ClientList *list = (*_dict)[ _signal ];
    if( list == 0L )
    {
      kdebug( KDEBUG_WARN, 8003, "Unknown signal %s", _signal );
      kdebug( KDEBUG_WARN, 8003, "Please insert SIGNAL_IMPL( \"%s\" ); in your "
	                         "constructor or choose a valid signal", _signal );
      assert( 0 );
    }

    QListIterator<KOMBase::Client> it( *list );
    for ( ; it.current(); ++it )
    {
        CORBA::Request_var _req = it.current()->obj->_request( it.current()->function.data() );
	_req->add_in_arg() <<= (P1)_p1;
	_req->add_in_arg() <<= (P2)_p2;
	_req->result()->value()->type( CORBA::_tc_void );
	_req->invoke();
	// if( CORBA::Exception *_ex = _req->env()->exception() )
	// mico_throw( *_ex );
    }
}

#define SIGNAL_CALL0( _sig ) signal_call0( _sig, &m_dctSignals )
#define SIGNAL_CALL1( _sig, _p1 ) signal_call1( _sig, &m_dctSignals, _p1 )
#define SIGNAL_CALL2( _sig, _p1, _p2 ) signal_call2( _sig, &m_dctSignals, _p1, _p2 )

typedef KOMVar<KOM::Base> Base_ref;

void sdeb( const char *msg, ... );
void edeb( const char *msg, ... );
void mdeb( const char *msg, ... );

#endif
