/* This file is part of the KDE project
   Copyright (C) 1998, 1999 Torben Weis <weis@kde.org>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
 
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.
 
   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/     

#ifndef __kom_component_h__
#define __kom_component_h__

#include <qvaluelist.h>

#include "kom.h"
#include "komBase.h"
#include "komVar.h"
#include "komContainer.h"

class KOMComponent;

typedef KOMVar<KOM::Aggregate> Aggregate_ref;
typedef KOMVar<KOM::Plugin> Plugin_ref;

CORBA::Boolean operator == ( CORBA::String_var&, const char* );

typedef CORBA::Object_ptr (*KOMAggregateFunc)( KOMComponent* );
typedef KOM::Plugin_ptr (*KOMPluginFunc)( KOMComponent* );

/**
 *  KOMMBuiltinAggregate struct.
 */
struct KOMBuiltinAggregate
{
  KOMBuiltinAggregate() {}

  /**
   *  Constructor.
   */
  KOMBuiltinAggregate( KOMAggregateFunc _func,
		       const KOM::InterfaceSeq& _required,
		       const KOM::InterfaceSeq& _provided,
                       KOM::ID _id )
  {
     activator = _func;
     requiredInterfaces = _required;
     providedInterfaces = _provided;
     id = _id;
  }

  /**
   *  Constructor.
   *
   *  @param _obj      Object.
   *  @param _required List (sequence) of all required interfaces.
   *  @param _provided List (sequence) of all provided interfaces.
   *  @param _id       KOM::ID of the aggregate.
   */
  KOMBuiltinAggregate( CORBA::Object_ptr _obj,
		       const KOM::InterfaceSeq& _required,
		       const KOM::InterfaceSeq& _provided,
		       KOM::ID _id )
  {
      activator = 0L;
      aggregate = CORBA::Object::_duplicate( _obj );
      requiredInterfaces = _required;
      providedInterfaces = _provided;
      id = _id;
  }
  
  /**
   *  Retrieves a reference.
   *
   *  @param _comp Component.
   *
   *  @return Object.
   */
  CORBA::Object_ptr ref( KOMComponent* _comp )
  {
    if( CORBA::is_nil( aggregate ) &&
      ( activator != 0L ) )
    {
      aggregate = activator( _comp );
    }
    return CORBA::Object::_duplicate( aggregate );
  }
  
  KOMAggregateFunc activator;
  CORBA::Object_var aggregate;
  KOM::InterfaceSeq requiredInterfaces;
  KOM::InterfaceSeq providedInterfaces;
  KOM::ID id;
  bool composed;
  bool precomposed;
};

/**
 *  KOMDynamicAggregate struct.
 */
struct KOMDynamicAggregate
{
  /**
   *  Constructor.
   */
  KOMDynamicAggregate() {}

  /**
   *  Constructor.
   */
  KOMDynamicAggregate( KOM::AggregateFactory_ptr _factory, 
		       const KOM::InterfaceSeq& _required, 
		       const KOM::InterfaceSeq& _provided, 
		       CORBA::Boolean _activate, KOM::ID _id )
  {
      factory = KOM::AggregateFactory::_duplicate( _factory );
      id = _id;
      composed = false;
      requiredInterfaces = _required;
      activate = _activate;
      precomposed = false;
      providedInterfaces = _provided;
  }
  
  /**
   *  ref
   */
  KOM::Aggregate_ptr ref( KOM::Component_ptr _comp )
  {
    if( !composed )
      return KOM::Aggregate::_nil();

    if( CORBA::is_nil( aggregate ) )
      aggregate = factory->create( _comp );

    return KOM::Aggregate::_duplicate( aggregate );
  }

  /**
   *  start
   */
  void start( KOM::Component_ptr _comp )
  {
    if( !composed )
      return;

    if( CORBA::is_nil( aggregate ) )
      aggregate = factory->create( _comp );
  }
  
  KOM::AggregateFactory_var factory;
  Aggregate_ref aggregate;
  KOM::ID id;
  bool composed;
  bool precomposed;
  KOM::InterfaceSeq requiredInterfaces;
  KOM::InterfaceSeq providedInterfaces;
  CORBA::Boolean activate;
};

/**
 *  KOMDerivedInterface struct.
 */
struct KOMDerivedInterface
{
  KOMDerivedInterface() {}

  /**
   *  Constructor.
   *
   *  @param _interface The interface.
   */
  KOMDerivedInterface( const char* _interface )
  {
    iface = CORBA::string_dup( _interface );
  }
  
  CORBA::String_var iface;
};

/**
 *  KOMBuiltinPlugin struct
 */
struct KOMBuiltinPlugin
{

  /**
   *  Constuctor.
   */
  KOMBuiltinPlugin() {}

  /**
   *  Constructor.
   *
   *  @param _func             KOMPluginFunc.
   *  @param _required         List (sequence) of all required interfaces.
   *  @param _required_plugins List (sequence) of all required plugin interfaces.
   *  @param _provided         List (sequence) of all provided interfaces.
   *  @param _id               ID.
   */
  KOMBuiltinPlugin( KOMPluginFunc _func,
                    const KOM::InterfaceSeq& _required,
		    const KOM::InterfaceSeq& _required_plugins, 
		    const KOM::InterfaceSeq& _provided, 
		    KOM::ID _id )
  {
    activator = _func;
    requiredInterfaces = _required;
    requiredPluginInterfaces = _required_plugins;
    providedInterfaces = _provided;
    id = _id;
  }

  /**
   *  ref.
   *
   *  @param _comp Component.
   */
  KOM::Plugin_ptr ref( KOMComponent* _comp )
  {
    if( CORBA::is_nil( plugin ) )
      plugin = activator( _comp );

    return KOM::Plugin::_duplicate( plugin );
  }
  
  KOMPluginFunc activator;
  KOM::Plugin_var plugin;
  KOM::InterfaceSeq requiredInterfaces;
  KOM::InterfaceSeq requiredPluginInterfaces;
  KOM::InterfaceSeq providedInterfaces;
  KOM::ID id;
  bool composed;
  bool precomposed;
};

/**
 *  KOMDynamicPlugin struct
 */
struct KOMDynamicPlugin
{
  KOMDynamicPlugin() {}

  /**
   *  Constructor.
   *
   *  @param _factory          PluginFactroyPtr.
   *  @param _required         List (sequence) of required interfaces.
   *  @param _required_plugins List (sequence) of required plugin interfaces.
   *  @param _provided         List (sequence) of provided interfaces.
   *  @param _activate         Boolean.
   *  @param _id               ID.
   */
  KOMDynamicPlugin( KOM::PluginFactory_ptr _factory,
		    const KOM::InterfaceSeq& _required,
		    const KOM::InterfaceSeq& _required_plugins, 
		    const KOM::InterfaceSeq& _provided, 
		    CORBA::Boolean _activate,
                    KOM::ID _id )
  {
    factory = KOM::PluginFactory::_duplicate( _factory );
    id = _id;
    composed = false;
    requiredInterfaces = _required;
    activate = _activate;
    precomposed = false;
    requiredPluginInterfaces = _required_plugins;
    providedInterfaces = _provided;
  }

  /**
   *  Retrieves.
   *
   *  @param _comp The parent component object.
   *
   *  @return Pointer.
   */
  KOM::Plugin_ptr ref( KOM::Component_ptr _comp )
  {
    if ( !composed )
      return KOM::Plugin::_nil();

    if ( CORBA::is_nil( plugin ) )
      plugin = factory->create( _comp );

    return KOM::Plugin::_duplicate( plugin );
  }

  /**
   *  Starts.
   *
   *  @param _comp A component object.
   */
  void start( KOM::Component_ptr _comp )
  {
    if ( !composed )
      return;

    if ( CORBA::is_nil( plugin ) )
      plugin = factory->create( _comp );
  }

  KOM::PluginFactory_var factory;
  Plugin_ref plugin;
  KOM::ID id;
  bool composed;
  bool precomposed;
  KOM::InterfaceSeq requiredInterfaces;
  KOM::InterfaceSeq requiredPluginInterfaces;
  KOM::InterfaceSeq providedInterfaces;
  CORBA::Boolean activate;
};

#define ADD_INTERFACE( name ) m_lstDerivedInterfaces.append( name );

/**
 *  The KOMComponent class represents a component.
 *
 *  The Component interface, being derived from the Base interface, additionally provides
 *  a kind of small interface repository for only this component, combined with the possibility
 *  to provide new interfaces by dynamic aggregation and a standard way to add plugin
 *  components. This gives CORBA objects the possibility to enhance ther functionality at
 *  run-time.
 *
 *  Ther are five kinds of interfaces:
 *
 *  @li builtin interfaces (derived interfaces)
 *  @li builtin aggregate interfaces
 *  @li dynamic aggregate interfaces
 *  @li builtin plugin interfaces
 *  @li dynamic plugin interfaces
 *
 *  @see KOMAggregate, KOMPlugin
 *
 *  @short A component with the possibility for dynamic extensions.
 */
class KOMComponent : virtual public KOMBase, virtual public KOM::Component_skel
{

public:

  /**
   *  Constructor.
   */
  KOMComponent();

  /**
   *  Destructor.
   */
  ~KOMComponent();
  
  /**
   *  Adds an aggregate to the KOM::Component object. An aggregate provides new interfaces
   *  to the component.
   *
   *  @param _factory  AggregateFactoryPtr.
   *  @param _required List (sequence) of required interfaces.
   *  @param _provided List (sequence) of provided interfaces.
   *  @param _activate Boolean.
   *
   *  @return Returns the ID of the added aggregate.
   *
   *  @see #removeAggregate
   */
  virtual KOM::ID addAggregate( KOM::AggregateFactory_ptr _factory,
				const KOM::InterfaceSeq& _required,
				const KOM::InterfaceSeq& _provided,
				CORBA::Boolean _activate);

  /**
   *  Adds an aggregate to the KOM::Component class.
   *
   *  @param _func     KOMAggregateFunc.
   *  @param _required List (sequence) of required interfaces.
   *  @param _provided List (sequence) of provided interfaces.
   *
   *  @return Returns the ID of the added aggregate.
   *
   *  @see #removeAggregate
   */
  virtual KOM::ID addAggregate( KOMAggregateFunc _func,
                                const KOM::InterfaceSeq& _required,
                                const KOM::InterfaceSeq& _provided );
 
  /**
   *  Adds an aggregate to the KOM::Component class.
   *
   *  @param _func     KOMAggregateFunc.
   *  @param _provided List (sequence) of provided interfaces.
   *
   *  @return Returns the ID of the added aggregate.
   *
   *  @see #removeAggregate
   */
  virtual KOM::ID addAggregate( KOMAggregateFunc _func,
                                const char* _provided );

  /**
   *  Adds an aggregate to the KOM::Component class.
   *
   *  @param _obj      Object.
   *  @param _required List (sequence) of required interfaces.
   *  @param _provided List (sequence) of provided interfaces.
   *
   *  @return Returns the ID of the added aggregate.
   *
   *  @see #removeAggregate
   */
  virtual KOM::ID addAggregate( CORBA::Object_ptr _obj,
                                const KOM::InterfaceSeq& _required,
                                const KOM::InterfaceSeq& _provided );
 
  /**
   *  Adds an aggregate to the KOM::Component class.
   *
   *  @param _obj      Object.
   *  @param _provided List (sequence) of provided interfaces.
   *
   *  @return Returns the ID of the added aggregate.
   *
   *  @see #removeAggregate
   */
  virtual KOM::ID addAggregate( CORBA::Object_ptr _obj,
                                const char* _provided );

  /**
   *  Removes an added aggregate from the component.
   *
   *  @param _id ID of the aggregate that will be removed from the component.
   *
   *  @see #addAggregate
   */
  virtual void removeAggregate( KOM::ID _id );

  /**
   *  Adds a plugin to the KOM::Component object. A plugin can change the implementation of
   *  an interface the component provides.
   *
   *  @param _factory          PluginFactoryPtr.
   *  @param _required         List (sequence) of required interfaces.
   *  @param _required_plugins List (sequence) of required plugin interfaces.
   *  @param _provided         List (sequence) of provided plugin interfaces.
   *  @param _activate         Boolean.
   *
   *  @return Returns the ID of the added plugin.
   *
   *  @see #removePlugin, #getPlugin, #describePlugins, #composePlugins
   */
  virtual KOM::ID addPlugin( KOM::PluginFactory_ptr _factory,
			     const KOM::InterfaceSeq& _required,
			     const KOM::InterfaceSeq& _required_plugins,
			     const KOM::InterfaceSeq& _provided,
			     CORBA::Boolean _activate);

  /**
   *  Removes an added plugin from the component.
   *
   *  @param _id ID of the plugin that will be removed from the component.
   *
   *  @see #addPlugin, #getPlugin, #describePlugins, #composePlugins
   */
  virtual void removePlugin( KOM::ID _id );
  
  /**
   *  Retrieves an interface with a given name.
   *
   *  @param _name Name of the interface that will be retrieved.
   *
   *  @return Pointer to the returned interface.
   *
   *  @see #interfaces, #supportsInterface, #getPluginInterface, #pluginInterfaces, #supportsPluginInterface
   */
  virtual CORBA::Object_ptr getInterface( const char* _name );

  /**
   *  Retrieves the interfaces provided by the KOM::Component class.
   *
   *  @return The provided interfaces.
   *
   *  @see #getInterfaces, #supportsInterface, #getPluginInterface, #pluginInterfaces, #supportsPluginInterface
   */
  virtual KOM::InterfaceSeq* interfaces();

  /**
   *  Checks if an interface is supported by the KOM::Component class.
   *
   *  @param _name Name of the interface.
   *
   *  @return Returns whether the interface is supported or not.
   *
   *  @see #interfaces, #getInterface, #getPluginInterface, #pluginInterfaces, #supportsPluginInterface
   */
  virtual CORBA::Boolean supportsInterface( const char* _name );

  /**
   *  Retrieves a pointer to an plugin interface given by the name.
   *
   *  @param _name Name of the plugin interface that shall be retrieved.
   *
   *  @return Pointer to the object that supports the plugin interface.
   *
   *  @see #interfaces, #getInterface, #supportsInterface, #pluginInterfaces, #supportsPluginInterface
   */
  virtual CORBA::Object_ptr getPluginInterface( const char* _name );

  /**
   *  Retrieves the plugin interfaces provided by the KOM::Component class.
   *
   *  @return The plugin interfaces.
   *
   *  @see #interfaces, #getInterface, #supportsInterface, #getPluginInterface, #supportsPluginInterface
   */
  virtual KOM::InterfaceSeq* pluginInterfaces();

  /**
   *  Checks if an plugin interface is supported by the KOM::Component class.
   *
   *  @param _name Name of the plugin interface.
   *
   *  @return Returns whether the plugin interface is supported or not.
   *
   *  @see #interfaces, #getInterface, #supportsInterface, #pluginInterfaces, #getPluginInterface
   */
  virtual CORBA::Boolean supportsPluginInterface( const char* _name );

  /**
   *  Retrives informations about all added plugins.
   *
   *  @return A Pointer to the informations.
   *
   *  @see #addPlugin, #removePlugin, #getPlugin, #composePlugins
   */
  virtual KOM::Component::PluginInfoSeq* describePlugins();

  /**
   *  Retrieves a plugin with a given ID.
   *
   *  @param _id The ID of the plugin that will be retrieved.
   *
   *  @return Pointer to a plugin.
   *
   *  @see #addPlugin, #removePlugin, #describePlugins, #composePlugins
   */
  virtual KOM::Plugin_ptr getPlugin( KOM::ID _id );
  
  // C++
  /**
   *  Clears all connections of the object.
   *
   *  @see KOMBase::cleanUp 
   */
  virtual void cleanUp();

protected:

  /**
   *  Retrievs all interfaces provided by plugins.
   *
   *  @param _include_precomposed Boolean.
   *
   *  @return Returns a list (sequence) of all provided plugin interfaces.
   */
  virtual KOM::InterfaceSeq* pluginInterfaces( bool _include_precomposed );

  /**
   *  Retrievs all provided interfaces.
   *
   *  @param _include_precomposed Boolean.
   *
   *  @return Returns a list (sequence) of all provided interfaces.
   */
  virtual KOM::InterfaceSeq* interfaces( bool _include_precomposed );

  /**
   *  Text.
   *
   *  @see #addAggregate, #removeAggregate, #composePlugins
   */
  virtual void composeAggregates();

  /**
   *  Text.
   *
   *  @see #addPlugin, #removePlugin, #getPlugin, #describePlugins
   */
  virtual void composePlugins();
  
  QValueList<KOMDerivedInterface> m_lstDerivedInterfaces;
  QValueList<KOMBuiltinAggregate> m_lstBuiltinAggregates;
  QValueList<KOMDynamicAggregate> m_lstDynamicAggregates;
  QValueList<KOMBuiltinPlugin> m_lstBuiltinPlugins;
  QValueList<KOMDynamicPlugin> m_lstDynamicPlugins;
  KOM::ID m_id;
};

typedef KOMVar<KOM::Component> Component_ref;

/**
 *  The KOMComponentFactory class.
 *
 *  You have to implement these on your own! Needed Methods are:
 *
 *  @li ::KOM::Aggregate_ptr create( Component_ptr core );
 */
class KOMComponentFactory : virtual public KOM::ComponentFactory_skel
{

public:

};

#endif
