/* This file is part of the KDE project
   Copyright (C) 1998, 1999 Torben Weis <weis@kde.org>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
 
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.
 
   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/     

#ifndef __kom_container_h__
#define __kom_container_h__

#include "kom.h"
#include "komBase.h"

#include <qmap.h>
#include <qcstring.h>

/**
 *  The KOMContainer class do something simple but extremely useful: They act as repository
 *  for container members. A container member structure consists of two elements:
 *
 *  @li the member name, a freely chooseable and unique string identifier
 *  @li the member object, a KOM::Base type
 *
 *  You can add, remove, replace, list and lookup container members or just clear the whole
 *  container. In addition the container emts signals whenever a member has been added or
 *  removed.
 *
 *  @short A KOMContainer acts as repository for KOM::Base types.
 */
class KOMContainer : virtual public KOMBase,
		     virtual public KOM::Container_skel
{

public:

  /**
   *  Constructor.
   */
  KOMContainer();

  /**
   *  Destructor.
   */
  virtual ~KOMContainer();
  
  /**
   *  Adds a KOM::Base type into the container.
   *
   *  @param _memb The object that will be added.
   *
   *  @see removeContainee, #containeeRemoved
   */
  virtual void addContainee( const KOM::Member& _memb );

  /**
   *  Removes a KOM::Base type given by a name from the container.
   *
   *  @param _name Name of the object that shall be removed.
   *
   * @see #addContainee, #containeeAdded
   */
  virtual void removeContainee( const char* _name );

  /**
   *  replaceContainee
   *
   *  @param _memb Member
   */
  virtual void replaceContainee( const KOM::Member& _memb );

  /**
   *  Retrieves alle containees that are in the container.
   *
   *  @return List (sequence) of all containees.
   */
  virtual KOM::MemberSeq* listContainees();

  /**
   *  This method retrieves a KOM::Base object when a containee is found by a given name 
   *  in the container.
   *
   *  @return A pointer to the found KOM::Base obejct.
   */
  virtual KOM::Base_ptr lookupContainee( const char* _name );

  /**
   *  This signal will be emitted when a containee will be added to the container.
   *
   *  @param _memb A reference to the KOM::Base object in the container.
   *
   *  @see #containeeRemoved
   */
  virtual void containeeAdded( const KOM::Member& _memb );

  /**
   *  This signal will be emitted when a containee will be removed from the container.
   *
   *  @param _memb A reference to the KOM::Base object in the container.
   *
   *  @see #containeeAdded
   */
  virtual void containeeRemoved( const KOM::Member& _memb );

  /**
   *  Removes alle containees from the container.
   */
  virtual void clear();
 
protected:

  /**
   *  leaveNotify
   */
  virtual void leaveNotify( KOM::Base_ptr _ptr );
  
  typedef QMap<QCString,KOM::Base_var> ContainerMap;
  
  ContainerMap m_mapContainees;
};

#endif
