/* This file is part of the KDE project
   Copyright (C) 1999 Simon Hausmann <hausmann@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/

#ifndef __komShutdownManager_h__
#define __komShutdownManager_h__

#include <qlist.h>
#include <qobject.h>
#include <qtimer.h>

class KOMBase;

/**
 *  This class will launch a (q)timer after all watched KOMBase (or derived)
 *  objects have been destroyed. When the timeout occurs then the ORB is shutted
 *  down.
 */
class KOMShutdownManager : public QObject
{
  Q_OBJECT

public:

  /**
   *  Constuctor.
   */
  KOMShutdownManager();

  /**
   *  Destructor.
   */
  virtual ~KOMShutdownManager();

  /**
   *  Allow/Prevent the manager to perform a shutdown after all watched objects are deleted.
   *
   *  @param _enableShutdown Indicates whether to allow or to prevent a shutdown.
   *
   *  @see #isEnabled, #watchObject
   */
  void enable( bool _enableShutdown ) { m_bEnable = _enableShutdown; }

  /**
   *  Check if a shutdown is enabled or not.
   *
   *  @return Returns whether TRUE if a shutdown is enabled or FALSE if not.
   *
   *  @see #enable
   */
  bool isEnabled() const { return m_bEnable; }

  /**
   *  Add the specified object to the list of watched objects. When all
   *  watched objects have been destroyed, then the shutdown timer is started.
   *
   *  @param _obj Pointer to the to be watched object.
   */    
  void watchObject( KOMBase* _obj );

  /**
   *  Do not call this function directly! It is internally called in the
   *  destructor of KOMBase.
   *
   *  @param _obj Pointer to the calling object.
   */
  void objectDestroyed( KOMBase* _obj );
  
  /**
   *  Specify the timeout for the shutdown timer. The default value choosen
   *  in the KOMShutdownManager constructor is one minute ( 60 * 1000 msec ) .
   *
   *  @param _msec Time in milliseconds until shutdown.
   *
   *  @see #shutdownTimeout, #slotTimeout
   */
  void setShutdownTimeout( int _msec ) { m_iTimeout = _msec; }

  /**
   *  Retrieves the value of the timeout.
   *
   *  @return Returns the timeout value.
   *
   *  @see #setShutdownTimeout, #slotTimeout
   */
  int shutdownTimeout() const { return m_iTimeout; }

  /**
   * @return Returns whether the manager is active and there are objects
   *         being watched.
   */
  bool isActive();

  /**
   *  Retrieves the pointer to the KOMShutdownManger object. Ther may only exist one
   *  shutdown manager over all.
   *
   *  @return Pointer to the KOMShutdownManger object.
   */
  static KOMShutdownManager* self();

protected slots:

  /**
   *  This slot is called internally from a QTimer object, when the timeout ends.
   *
   *  @see #setShutdownTimeout, #shutdownTimeout
   */
  void slotTimeout();

private:
  QList<KOMBase> m_lstObjs;
  QTimer* m_pTimer;
  int m_iTimeout;
  bool m_bEnable;
  
  static KOMShutdownManager *s_pSelf;
};

#endif

