#pragma interface

#ifndef __trader_h__
#define __trader_h__

#include <qstring.h>
#include <qstringlist.h>

#include <ksharedptr.h>
#include <kuserprofile.h>
#include <kservices.h>
#include <krun.h>
#include "kded.h"

class KServiceFactory;

/**
 * @short Abstract base class for the trader. It is used to hide whether a local
 *        or a remote trader is used.
 */
class KTrader
{
public:
  // A list of services
  typedef QValueList<KService::Ptr> OfferList;

  /**
   * Get the list of services matching the given servicetype
   */
  virtual OfferList query( const QString& _servicetype,
			   const QString& _constraint = QString::null,
			   const QString& _preferences = QString::null ) const = 0;
			   
  virtual OfferList listServices() = 0;
  
  virtual KService::Ptr serviceByName( const QString &name ) = 0;
};

/**
 * @short Implementation of the local trader.
 */
class KLocalTrader : public KTrader
{
public:
  KLocalTrader();
  virtual ~KLocalTrader();

  virtual OfferList query( const QString& _servicetype,
			   const QString& _constraint = QString::null,
			   const QString& _preferences = QString::null) const;

  virtual OfferList listServices();

  virtual KService::Ptr serviceByName( const QString &name );
  
  void loadRegistry();
  
  KServiceFactory *serviceFactory() const { return m_pServiceFactory; }
  
private:
  KServiceFactory *m_pServiceFactory;
};

/**
 * @short Implementation of the remote trader.
 */
class KRemoteTrader : public KTrader
{
public:
  KRemoteTrader( KDED::Trader_ptr remoteTrader );
  virtual ~KRemoteTrader();

  virtual OfferList query( const QString& _servicetype,
			   const QString& _constraint = QString::null,
			   const QString& _preferences = QString::null) const;

  virtual OfferList listServices();

  virtual KService::Ptr serviceByName( const QString &name );

private:
  KDED::Trader_var m_vTrader;
};

/**
 * This class is an reimplementation of the KServiceProvider class, used by
 * KRun (libkio) to receive KService data for a certain servicetype. The
 * original implementation uses KServiceTypeProfile to actually get KService
 * data, whereas this class uses KTrader, which is more efficient for most
 * applications, since it doesn't require you to read the whole registry just
 * in order to be able to use KRun.
 * If you want to use this class, then simply create a single instance of it 
 * somewhere in your code (preferably somewhere in main() ) , just like:
 * KTraderServiceProvider traderProvider;
 * That's it :-) , oh, and make sure you have a KdedInstance!!
 */
class KTraderServiceProvider : public KServiceProvider
{
public:
  KTraderServiceProvider() {} ;
  virtual ~KTraderServiceProvider() {} ;
  
  virtual KService::Ptr serviceByServiceType( const char *mime_type );
  virtual KService::Ptr serviceByName( const QString &name );
};


#endif
