/* This file is part of the KDE project
   Copyright (C) 1998, 1999 Torben Weis <weis@kde.org>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
 
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.
 
   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/     

#ifndef __op_MainWindow_h__
#define __op_MainWindow_h__

#include <ktmainwindow.h>
#include <qintdict.h>

#include <opToolBar.h>
#include <opStatusBar.h>

class OPMainWindowIf;
class OPMenuBar;
class OPMenuBarManager;
class OPToolBarFactory;
class OPToolBarManager;
class OPStatusBarManager;
class OPToolBar;
class OPStatusBar;

/**
 *  The OPMainWindow class.
 */
class OPMainWindow : public KTMainWindow
{
  Q_OBJECT

  friend OPMainWindowIf;
  
public:

  /**
   *  Constructor.
   *
   *  @param _name Name.
   */
  OPMainWindow( const char* _name = 0L );

  /**
   *  Destructor.
   */
  ~OPMainWindow();

  /**
   *  Retrieves the interface of OPMainWindow.
   *
   *  @return A pointer to the interface object.
   */
  virtual OPMainWindowIf* interface();

  /**
   *  The Toolbars created with this function are always persistent and not
   *  managed by the @ref OPToolBarManager. This function is a replacement for
   *  @ref #toolBar which just returns an ordinary @ref KToolBar.
   *
   *  @param _id ID of the toolbar.
   *
   *  @return Pointer to a toolbar object with a given ID.
   */
  virtual OPToolBar* opToolBar( int _id = 0 );

  /**
   *  Do NOT use together with the @ref OPMenuBarManager.
   *  This function is a replacement for @ref #menuBar which just returns an ordinary @ref KMenuBar.
   *
   *  @return Pointer to the menubar object.
   */
  virtual OPMenuBar* opMenuBar();
  
  /**
   *  Used in conjunction with @ref OPMenuBarManager
   *
   *  @param _menubar Menubar.
   *
   *  @see createHelpMenu
   */
  virtual void createFileMenu( OPMenuBar* _menubar );

  /**
   *  Used in conjunction with @ref OPMenuBarManager.
   *
   *  @param _menubar Menubar.
   *
   *  @see createFileMenu
   */
  virtual void createHelpMenu( OPMenuBar* _menubar );

  /**
   *  @return A pointer to the @ref OPMenuBarManager.
   *          If there is yet no one, then a OPMenuBarManager
   *          is created. When using the menu bar manger you should implement
   *          @ref #createFileMenu and @ref #createHelpMenu. The manager will
   *          call this function to tell you, when you have to create or release
   *          our menu.
   *
   *  @return Pointer to the OPMenuBarManager object
   *
   *  @see hasMenuBarManager
   */
  virtual OPMenuBarManager* menuBarManager();

  /**
   *  Checks whether the mainwindow has a MenuBarManager or not..
   *
   *  @return Indicates if the MenuBarManager exists.
   *
   *  @see menuBarManager
   */
  virtual bool hasMenuBarManager()
  { return ( m_pMenuBarManager != 0L ); }

  /**
   *  Retrieves the OPToolBarManager of the mainwindow.
   *
   *  @return Pointer to the OPToolBarManager object.
   *
   *  @see hasToolBarManager
   */
  virtual OPToolBarManager* toolBarManager();

  /**
   *  Checks whether the mainwindow has a ToolBarManager or not.
   *
   *  @return Indicates if the ToolBarManager exists.
   *
   *  @see toolBarManager
   */
  virtual bool hasToolBarManager()
  { return ( m_pToolBarManager != 0L ); }

  /**
   *  Retrieves the OPToolBarFactory of the mainwindow.
   *
   *  @return Pointer to the OPToolBarFactory object.
   *
   *  @see hasToolBarFactory
   */
  virtual OPToolBarFactory* toolBarFactory();

  /**
   *  Checks whether the mainwindow has a ToolBarFactory or not.
   *
   *  @return Indicates if the ToolBarFactory exists.
   *
   *  @see toolBarFactory
   */
  virtual bool hasToolBarFactory()
  { return ( m_pToolBarFactory != 0L ); }
  
  /**
   *  Retrieves the OPStatusBarManager of the mainwindow.
   *
   *  @return Pointer to the OPStatusBarManager object.
   *
   *  @see hasStatusBarManager
   */
  virtual OPStatusBarManager* statusBarManager();

  /**
   *  Checks whether the mainwindow has a StatusBarManager or not.
   *
   *  @return Indicates if the StatusBarManager exists.
   *
   *  @see statusBarManager
   */
  virtual bool hasStatusBarManager()
  { return ( m_pStatusBarManager != 0L ); }

  /**
   *  If AutoCaption mode is enabled, the main window will automatically set
   *  the main window caption, corresponding to the caption of the active part.
   *  If AutoCaption mode is disabled the window caption will not change when
   *  the active part changes.
   */
  virtual bool autoCaptionMode()
  { return m_bAutoCaption; }
 
  /**
   *  setAutoCaption.
   *
   *  @param _flag Boolean.
   */
  virtual void setAutoCaption( bool _flag )
  { m_bAutoCaption = _flag; }
  
signals:  

  /**
   *  @param _new_part_id New Part.
   *  @param _old_part_id Old Part.
   */
  void activePartChanged( unsigned long _new_part_id, unsigned long _old_part_id );
  
protected:
 
  /**
   *  emitActivePartChanged.
   *
   *  @param _new_part_id New Part.
   *  @param _old_part_id Old Part.
   */
  void emitActivePartChanged( unsigned long _new_part_id, unsigned long _old_part_id );

  /**
   *  showEvent.
   *
   *  @param _event Event.
   */
  void showEvent( QShowEvent* _event );
 
  /**
   *  hideEvent.
   *
   *  @param _event Event.
   */
  void hideEvent( QHideEvent* _event );

  bool m_bAutoCaption;
  OPMainWindowIf* m_pInterface;
  OPMenuBar* m_pMenuBar;
  OPMenuBarManager* m_pMenuBarManager;
  OPToolBarManager* m_pToolBarManager;
  OPToolBarFactory* m_pToolBarFactory;
  OPStatusBarManager* m_pStatusBarManager;
  QIntDict<OPToolBar> m_mapToolBars;
};

#endif
