/* Copyright (C) 1983-1999 Applix, Inc.
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.
   
   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
   02111-1307, USA.
   
   Applix Inc. is the original author of this material.  Please send
   generally useful enhancements to Applix in the form of all modified 
   source files including an explanation of the changes.  For more
   information see our website at www.applixware.org.
   
   Do not remove or modify the license information above or the following
   Applix source control information from this file.
   
   Applix filename: cpr_symbol.c
   Applix ASC version: 2
   Date: Sun Jan 24 21:13:18 1999
*/
/* $Id: cpr_symbol.c,v 4.1 1994/10/12 14:12:48 cthuang Exp $
 *
 * Implements a symbol table abstract data type.
 */
#include <stdio.h>
#include "cproto.h"
#include "cpr_symbol.h"

static unsigned hash ARGS((char *name));
static Symbol *search_symbol_list ARGS((Symbol *list, char *name));

/* Create a symbol table.
 * Return a pointer to the symbol table or NULL if an error occurs.
 */
SymbolTable *
new_symbol_table ()
{
    SymbolTable *symtab;
    int i;

    if ((symtab = ALLOC(SymbolTable)) != NULL) {
	for (i = 0; i < SYM_MAX_HASH; ++i)
	    symtab->bucket[i] = NULL;
    }
    return symtab;
}


/* Free the memory allocated to the symbol table.
 */
void
free_symbol_table (symtab)
SymbolTable *symtab;
{
    int i;
    Symbol *sym, *next;

    for (i = 0; i < SYM_MAX_HASH; ++i) {
	sym = symtab->bucket[i];
	while (sym != NULL) {
	    next = sym->next;
	    if (sym->name  != 0) free(sym->name);
	    if (sym->value != 0) free(sym->value);
	    free(sym);
	    sym = next;
	}
    }
    free(symtab);
}


/* This is a simple hash function mapping a symbol name to a hash bucket. */

static unsigned
hash (name)
char *name;
{
    char *s;
    unsigned h;

    h = 0;
    s = name;
    while (*s != '\0')
	h = (h << 1) ^ *s++;
    return h % SYM_MAX_HASH;
}


/* Search the list of symbols <list> for the symbol <name>.
 * Return a pointer to the symbol or NULL if not found.
 */
static Symbol *
search_symbol_list (list, name)
Symbol *list;
char *name;
{
    Symbol *sym;

    for (sym = list; sym != NULL; sym = sym->next) {
	if (strcmp(sym->name, name) == 0)
	    return sym;
    }
    return NULL;
}


/* Look for symbol <name> in symbol table <symtab>.
 * Return a pointer to the symbol or NULL if not found.
 */
Symbol *
find_symbol (symtab, name)
SymbolTable *symtab;
char *name;
{
    return search_symbol_list(symtab->bucket[hash(name)], name);
}


/* If the symbol <name> does not already exist in symbol table <symtab>,
 * then add the symbol to the symbol table.
 * Return a pointer to the symbol.
 */
Symbol *
new_symbol (symtab, name, value, flags)
SymbolTable *symtab;	/* symbol table */
char *name;		/* symbol name */
char *value;		/* symbol value */
int flags;		/* symbol attributes */
{
    Symbol *sym;
    int i;

    if ((sym = find_symbol(symtab, name)) == NULL) {
	sym = ALLOC(Symbol);
	sym->name = xstrdup(name);
	i = hash(name);
	sym->next = symtab->bucket[i];
	symtab->bucket[i] = sym;
    } else {
	free(sym->value);
    }
    sym->value = (value != NULL) ? xstrdup(value) : NULL;
    sym->flags = flags;
    return sym;
}
