/* Copyright (C) 1983-1999 Applix, Inc.
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.
   
   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
   02111-1307, USA.
   
   Applix Inc. is the original author of this material.  Please send
   generally useful enhancements to Applix in the form of all modified 
   source files including an explanation of the changes.  For more
   information see our website at www.applixware.org.
   
   Do not remove or modify the license information above or the following
   Applix source control information from this file.
   
   Applix filename: AxEditMask.C
   Applix ASC version: 2
   Date: Fri Feb  5 21:11:58 1999
*/
#include <string.h>
#include <ctype.h>

#include <AxEditMask.h>

#ifndef true
#define true 1
#endif
#ifndef false
#define false 0
#endif

// This file is a parallel of AxEditMask.java - please see that file also.
//

// the "masked" entry field restricts input according to its
// edit_mask member.  This file defines the edit mask itself.
//
// The edit_mask is a character string which contains 3 types of
// characters: placeholders, literals, and separators.
//
// A placeholder character represents a class of acceptable input
// characters at that position in the entry field.  For instance,
// the '#' in edit_mask[0] means that any of the set:
// { 0-9, plus, minus, decimal, space } are accepted at index 0 of
// the entry field value.  There are 7 defined placeholder characters:
// 9#A?&!~    One placeholder char can accept exactly one input char
// from the class of characters it accepts.
//
// Literals are any characters in the edit_mask which aren't placeholders,
// and are either one of the predefined literal chars, or are escaped.
// the predefined literal chars are:  $()-
//
// Any characters which are neither placeholders or literals will be
// stored in the mask as spacechars.  Such positions in the mask won't
// accept input.  Strictly speaking, an edit mask that contains these
// characters isn't valid (but this class will tolerate it anyway.)
// Documentation should stress that any literals which aren't in the
// predefined set must be preceded by a backslash in the mask definition --
// this is the best way to avoid conflicts in the future, should new
// placeholders be defined.
//

int AxEditMask::in_array(char *inarray, char ccode) {
	int ix;
	for (ix = 0; 0 != inarray[ix]; ix++) {
		if (inarray[ix] == ccode) {
			return true;
		}
	}
	return false;
}

// Return whether the mchars[] character at index 'dex' is an
// escape character (that is, a backslash which isn't itself escaped).
// This function must be called in the context of a forward iteration
// thru the mchars[]; param last_escape_dex is < dex during scan.
//
int AxEditMask::is_escape_char(char *mchars, int dex, int last_escape_dex) {
	if ('\\' == mchars[dex] && (dex == 0 || last_escape_dex != dex - 1)) {
		return true;
	}
	return false;
}

// convert the passed plain index (user-visible entry field index)
// to its corresponding index in the mchars.  The two indices can
// differ because of the presence of escape chars '\' in the mchars.
//
int AxEditMask::to_mask_dex(char *mchars, int mcharslen, int plainDex) {
	if (plainDex >= mcharslen) {
		plainDex = mcharslen - 1;
	}
	int ix;
	int skip = 0;
	int last_escape_dex = -1;
	for (ix = 0; ix < mcharslen; ix++) {
		if (is_escape_char(mchars, ix, last_escape_dex)) {
			// mchars[ix] is a non-escaped backslash; skip
			//
			last_escape_dex = ix;
			skip++;
		}
		if (ix >= plainDex + skip) {
			break;
		}
	}
	return ix < mcharslen ? ix : mcharslen - 1;
}

// return whether the mchars character at index 'dex' is a
// placeholder character.  This is defined as (char's code is
// in hardcoded placeholder charcodes) && (char is not escaped)
//
int AxEditMask::is_placeholder_index(char *mchars, int mcharslen, int dex) {
	if (dex >= mcharslen) {
		dex = mcharslen - 1;
	}
	int ix;
	int last_escape_dex = -1;
	for (ix = 0; ix < mcharslen; ix++) {
		if (is_escape_char(mchars, ix, last_escape_dex)) {
			last_escape_dex = ix;
		}
		if (dex == ix) {
			if (in_array(hard_placeholders, mchars[ix])) {
				return 0 == ix || last_escape_dex != ix - 1;
			}
			return false;
		}
	}
	return false;
}

// return whether the mchars character at index 'dex' is a
// literal character.  This is defined as (char is escaped) ||
// (char is in hardcoded literal charcodes)
//
int AxEditMask::is_literal_index(char *mchars, int mcharslen, int dex) {
	if (dex >= mcharslen) {
		dex = mcharslen - 1;
	}
	int last_escape_dex = -1;
	for (int ix = 0; ix < mcharslen; ix++) {
		if (is_escape_char(mchars, ix, last_escape_dex)) {
			last_escape_dex = ix;
		}
		if (dex == ix) {
			if (in_array(hard_literals, mchars[ix]) ||
				(ix > 0 && last_escape_dex == ix - 1)) {

				return true;
			}
			return false;
		}
	}
	return false;
}

int AxEditMask::placeholders_exist() {
	int ix;
	for (ix = 0; ix < edit_mask_len; ix++) {
		if (placeholders[ix]) {
			return true;
		}
	}
	return false;    // this mask can't be typed into at all..
}

// Return whether the 'inchar' is acceptable to the placeholder
// at mask index 'dex'. This is the quick-n-cheesy way to do this.
// Zero is returned if the input is unacceptable.
//
int AxEditMask::accepts_char (char inchar, int dex) {
	if (dex < 0 || dex >= edit_mask_len) {
		return 0;
	}
	if (!placeholders[dex]) {
		if (inchar == edit_mask[dex]) {
			return inchar;
		}
		else {
			return 0;
		}
	}
	int accepted_char = 0;                  // init to 'unaccepted'

	switch (edit_mask[dex]) {
	case '9':     // 0-9
		if (isdigit((int) inchar)) {
			accepted_char = (int) inchar;
		}
		break;

	case '#':     // 0-9  +  -  .  spacechar
		if (isdigit((int) inchar) || '+' == inchar ||
			'-' == inchar || '.' == inchar || ' ' == inchar) {

			accepted_char = (int) inchar;
		}
		break;

	case 'A':     // a-z  A-Z  0-9
		if (islower((int) inchar) ||
			isupper((int) inchar) ||
			isdigit((int) inchar)) {

			accepted_char = (int) inchar;
		}
		break;

	case '?':     // a-z  A-Z
		if (islower((int) inchar) ||
			isupper((int) inchar)) {

			accepted_char = (int) inchar;
		}
		break;

	case '&':     // any charcode in the "latin footprint"
		if((inchar > 31 && inchar < 127) ||
		   ((unsigned char)inchar > 159) && ((unsigned char)inchar < 257)) {
			accepted_char = (int) inchar;
		}
		break;

	case '!':     // a-z  A-Z  spacechar (do upcase on alpha char)
		if (islower((int) inchar)) {
			accepted_char = toupper((int) inchar);
		}
		else if (isupper((int) inchar) || ' ' == inchar) {
			accepted_char = (int) inchar;
		}
		break;

	case '~':     // a-z  A-Z  spacechar (do downcase on alpha char)
		if (isupper((int) inchar)) {
			accepted_char = tolower((int) inchar);
		}
		else if (islower((int) inchar) || ' ' == inchar) {
			accepted_char = (int) inchar;
		}
		break;

	}
	return accepted_char;
}

AxEditMask::AxEditMask (char *inmask) {
	int ix, inmask_len;
	if (0 == inmask || 0 == *inmask) {
		edit_mask = 0;
		edit_mask_len = 0;
		placeholders = 0;
		literals = 0;
		hard_placeholders = 0;
		hard_literals = 0;
		return;
	}
	hard_placeholders = "9#A?&!~";
	hard_literals = "$()-";

	// init mask_len as inmask's length minus the number of
	// "non-backslashed" backslashes.  Such escape chars are
	// used for supplying literals outside the base set.
	//
	edit_mask_len = inmask_len = strlen(inmask);
	int last_escape_dex = -1;
	for (ix = 0; ix < edit_mask_len; ix++) {
		if (is_escape_char(inmask, ix, last_escape_dex)) {
			last_escape_dex = ix;
			edit_mask_len--;
		}
	}

	// copy the inmask chars, minus the escape chars.
	// while doing, mark whether the characters are
	// placeholders, literals, or neither
	//
	edit_mask = new char[edit_mask_len + 1];
	placeholders = new int[edit_mask_len];
	literals = new int[edit_mask_len];
	for (ix = 0; ix < edit_mask_len; ix++) {
		int mdex = to_mask_dex(inmask, inmask_len, ix);
		edit_mask[ix] = inmask[mdex];
		if (is_placeholder_index(inmask, inmask_len, mdex)) {
			placeholders[ix] = true;
			literals[ix] = false;
		}
		else if (is_literal_index(inmask, inmask_len, mdex)) {
			placeholders[ix] = false;
			literals[ix] = true;
		}
		else {
			placeholders[ix] = literals[ix] = false;
		}
	}
	edit_mask[edit_mask_len] = 0;  // just being polite..
}

AxEditMask::~AxEditMask() {
	if (0 != edit_mask) {
		delete[] edit_mask;
	}
	if (0 != placeholders) {
		delete[] placeholders;
	}
	if (0 != literals) {
		delete[] literals;
	}
}

// Return whether passed inmask differs from current edit_mask
//
int AxEditMask::mask_differs(char *inmask) {
	if (0 == inmask && 0 == edit_mask) {
		return true;
	}
	else if (0 != inmask && 0 != edit_mask) {
		return strcmp(inmask, edit_mask);
	}
	else {
		return false;
	}
}

// Return the length of the edit mask.  The mask-constrained
// text field will always have exactly this text length.
//
int AxEditMask::mask_length() {
	return edit_mask_len;
}

// Return the string representing the "empty" value of the field.
// Caller must delete[] the returned string.
//
char * AxEditMask::initial_value () {
	if (0 == edit_mask || 0 == edit_mask_len) {
		return 0;
	}
	int ix;
	char *ichars = new char[edit_mask_len + 1];
	ichars[edit_mask_len] = 0;
	for (ix = 0; ix < edit_mask_len; ix++) {
		if (literals[ix]) {
			ichars[ix] = edit_mask[ix];
		}
		else {
			ichars[ix] = ' ';
		}
	}
	ichars[edit_mask_len] = 0;    // fulfill W_ENTER assumption
	return ichars;
}

// Return the next higher legal cursor index.
//
// param curdex:  the current cursor index in the field.
//
int AxEditMask::increment_cursor_position (int curdex) {
	if (!placeholders_exist()) {
		return curdex;
	}
	int mlen = edit_mask_len;
	int ix = curdex < 0 ? 0 : curdex;
	ix = ix < mlen ? ix : mlen - 1;
	ix++;
	while (ix < mlen && !placeholders[ix]) {
		ix++;
	}
	if (ix == mlen && !placeholders[ix - 1]) {
		ix = decrement_cursor_position(ix);
	}
	return (ix);
}

// Return the next lower legal cursor index.
//
// param curdex:  the current cursor index in the field.
//
int AxEditMask::decrement_cursor_position (int curdex) {
	if (!placeholders_exist()) {
		return curdex;
	}
	int mlen = edit_mask_len;
	int ix = curdex < 1 ? 1 : curdex;
	ix = ix <= mlen ? ix : mlen - 1;
	ix--;
	while (ix > 0 && !placeholders[ix]) {
		ix--;
	}
	if (ix == 0 && !placeholders[ix]) {
		ix = increment_cursor_position(ix);
	}
	return (ix);
}
	
// return the smallest legal cursor position
//
int AxEditMask::home_cursor() {
	if (placeholders[0]) {
		return 0;
	}
	else {
		return increment_cursor_position(0);
	}
}

// return the largest legal cursor position
//
int AxEditMask::end_cursor() {
	if (placeholders[edit_mask_len - 1]) {
		return edit_mask_len;
	}
	else {
		return decrement_cursor_position(edit_mask_len);
	}
}

// Perform "deletion" of characters over the indicated range.
// Passed curstring is modified in place -- its length is assumed to
// be >= edit_mask_len..
//
// param curstring:  the current value of the field, in its entirety
// param dex:        the starting index of the deletion
// param len:        the #chars to delete from curstring
//
// Note that "deleting" characters from the string means to change
// them to spaces if they occupy placeholder positions, do-nothing if
// they occupy literal positions.
//
void AxEditMask::deleteChars(char *curstring, int dex, int len) {
	if (0 == curstring) {
		return;
	}
	dex = dex < 0 ? 0 : dex;
	dex = dex < edit_mask_len ? dex : edit_mask_len - 1;
	if (dex + len > edit_mask_len) {
		len = edit_mask_len - dex - 1;
	}
	int lx = dex + len;
	for (int ix = dex; ix < lx; ix++) {
		if (placeholders[ix]) {
			curstring[ix] = ' ';
		}
		else {
			curstring[ix] = edit_mask[ix];
		}
	}
}

// Perform overwrite of the passed overstring onto the passed curstring.
// Param curstring is modified in place -- its length is assumed to
// be >= edit_mask_len
//
// param curstring:  the current value of the field, in its entirety
// param overstring: the incoming string segment, to apply to field at dex
// param overlen:    the length of overstring
// param dex:        the starting index of the overwrite
//
// Overwriting behaves according to a "keystroke entry" analog,
// that is, how would the field behave if each character of 'overstring'
// were typed in individually.
//
void AxEditMask::overwriteChars(char *curstring, char *overstring,
								int overlen, int dex) {
	if (0 == overstring) {
		return;
	}
	int  accepted_charcode;
	
	dex = dex < 0 ? 0 : dex;
	dex = dex < edit_mask_len ? dex : edit_mask_len - 1;

	int ox = 0;    // overstring index
	int nx = dex;  // curstring index
	for (ox = 0; ox < overlen; ox++) {
		if (nx >= edit_mask_len) {
			break;
		}
		if (placeholders[nx]) {
			accepted_charcode = accepts_char(overstring[ox], nx);
			if (0 != accepted_charcode) {
				curstring[nx] = (char) accepted_charcode;
				nx++;
			}
		}
		else if (overstring[ox] == curstring[nx]) {
			nx++;                          // literal-input ok if ==
		}
		else {
			int ixtmp = increment_cursor_position(nx);
			if (placeholders[ixtmp]) {
				accepted_charcode =
					accepts_char(overstring[ox], ixtmp);
				if (0 != accepted_charcode) {
					curstring[ixtmp] = (char) accepted_charcode;
					nx = ixtmp + 1;
				}
			}
		}
	}
}

// test whether the passed overstring will be fully accepted by
// the editmask.  Returns the number of characters of overstring that
// would be accepted
//
// param curstring:  the current field value, in its entirety
// param overstring: the incoming string, to test
// param overlen:    the length of overstring
// param dex:        the starting index of the "test overwrite"
//
int AxEditMask::testChars (char *curstring, char *overstring,
						   int overlen, int dex) {
	if (0 == overstring) {
		return 0;
	}
	int nx = dex;
	int num_accepted = 0;
	for (int ox = 0; ox < overlen; ox++) {
		if (nx >= edit_mask_len) {
			break;
		}
		if (placeholders[nx]) {
			if (0 != accepts_char(overstring[ox], nx)) {
				num_accepted++;
				nx++;
			}
		}
		else if (overstring[ox] == curstring[nx]) {
			num_accepted++;
			nx++;
		}
		else {
			int ixtmp = increment_cursor_position(nx);
			if (placeholders[ixtmp]) {
				if (0 != accepts_char(overstring[ox], ixtmp)) {
					num_accepted++;
					nx = ixtmp + 1;
				}
			}
		}
	}
	return num_accepted;
}

