/* Copyright (C) 1983-1999 Applix, Inc.
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.
   
   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
   02111-1307, USA.
   
   Applix Inc. is the original author of this material.  Please send
   generally useful enhancements to Applix in the form of all modified 
   source files including an explanation of the changes.  For more
   information see our website at www.applixware.org.
   
   Do not remove or modify the license information above or the following
   Applix source control information from this file.
   
   Applix filename: pfrast.c
   Applix ASC version: 19
   Date: Mon Nov 16 13:56:49 1998
*/
#include <stdio.h>
#include <pf.h>
#include <aos.h>
#include <paratm.h>
#include <paratmgr.h>
#include <devsetup.h>
#include <pfint.h>
#include <afileimg.h>
#include <basicwin.h>
#if THIMBLE
#include <fsrts.h>
#endif

typedef struct edge
{
	int		num_of_scanlines;
	int		starting_y;
	double	x;
	double	x_increment;
	struct	edge *next_edge;
} EDGE;

typedef struct buck
{
	int		num_of_edges;
	int		scanline_y;
	struct	edge *edge_list;
	struct	buck *next_buck;
} BUCKET;


#include <pfrast.p>

/*-----------------------------------------------------------------------------
	defines
-----------------------------------------------------------------------------*/
#define	TOP_BIT		0x80

#define	top(x,y)	((x)<(y) ? (x):(y))
#define	bottom(x,y)	((x)>(y) ? (x):(y))


/*-----------------------------------------------------------------------------
	static data
-----------------------------------------------------------------------------*/
static	int		first_bits;
static	int		min_scany, max_scany;
static	EDGE	*ael_list = NULL;
static	int		masks [8] =
	{0x80, 0x40, 0x20, 0x10, 0x08, 0x04, 0x02, 0x01};




/*-----------------------------------------------------------------------------
	   POLY_FILL (see pgon_fill() in gefilters)
	(See Foley and Van Dam for a complete description)

	Do a top (min) to bottom (max) scanline fill.
	For each edge in the polyline, take the min
	of the 2 y coordinates.	If there's already a
	bucket corresponding to that coordinate, add
	the edge to it else create a new bucket for
	that y coordinate.
	After all edges have been processed, scan from top
	to bottom, keeping the Active Edge List, a (sorted)
	list of all intersecting x coordinates at the current
	y coordinate.  At each scanline, update the AEL as
	necessary. Use the even-odd rule for drawing.
-----------------------------------------------------------------------------*/

void poly_fill(ptr1, ptr2, pol, boundingBox, miny, maxy, lined)
	void *ptr1;
	void *ptr2;
	poly_type *pol;
	TmPoint32 boundingBox[];
	int miny, maxy;
	void (*lined)();
{
	BUCKET *bucket, *tmpBkt;
	register EDGE *edge, *edge1, *edge2;
	int ael_cnt;
	register int ii;

	/* keep track of min and max scanlines that got changed */
	min_scany = 0;
   	max_scany = 50000;

	bucket = (BUCKET *)calloc(1,sizeof(BUCKET));  /* we need at least 1! */
	if (bucket == NULL)
		return;
	bucket->num_of_edges = 0;
	bucket->scanline_y = -1;

	/* store each edge in a bucket */
	for (ii=0; ii<pol->nPoints-1; ii++)
		add_to_bucket(&bucket, pol->points[ii], pol->points[ii+1]);

	/* plus one for my baby */
	add_to_bucket(&bucket, pol->points[ii], pol->points[0]);

	/* process bucket list */
	ael_cnt = 0;
	ael_list = NULL;   /* active edges list */


	/* top of polygon to bottom */
	for (ii=min_scany; ii<max_scany; ii++)
	{
		/* added buckets list to end of active list */
		if ((bucket) && (bucket->scanline_y == ii)) /* any new edges to add from the bucket? */
		{
			if (ael_list != NULL)
			{
				edge = ael_list;
				while (edge->next_edge != NULL)
					edge = edge->next_edge;
				edge->next_edge = bucket->edge_list;
				ael_cnt += bucket->num_of_edges;
			}
			else /* first time only to initialize ael_list */
			{
				ael_list = bucket->edge_list;
				ael_cnt = bucket->num_of_edges;
			}

			bsort();  /* sort here because we just added to ael_list */
			tmpBkt = bucket;
			bucket = bucket->next_buck;
			free(tmpBkt);
		}
		/* pull out pairs of edges to draw against */
		if ((edge1 = ael_list ) == NULL) /* head */
			continue; /* sanity check */
		edge2 = edge1->next_edge; /* first 2 edges */
		while (edge2 != NULL) /* walking along scanline in x */
		{
			/* fill between edge1 and edge2; */
			/* coordinates are converted to the 0..width, 0..height
			"image space" from the boundingBox[0,1].x,y line space. */
			(*lined)(ptr1, ptr2,
					 ((int)edge1->x - boundingBox[0].x), (ii - boundingBox[0].y),
					 ((int)edge2->x - boundingBox[0].x), (ii - boundingBox[0].y));

			/* increment line coords */
			(edge1->num_of_scanlines)--;  edge1->x += edge1->x_increment;
			(edge2->num_of_scanlines)--;  edge2->x += edge2->x_increment;

			/* get the next pair */
			edge1 = edge2->next_edge;
			if (edge1 != NULL)
				edge2 = edge1->next_edge;
			else
				break;
		}

		remove_dead_edges(&ael_cnt);

		/* sort here because decrementing x may have crossed lines */
		bsort();
	}
}

/*-----------------------------------------------------------------------------
	REMOVE DEAD EDGES
-----------------------------------------------------------------------------*/
static void remove_dead_edges(ael_cnt)
	 int *ael_cnt;
{
	EDGE *edge, *prev;

	edge = ael_list;
	while (edge->num_of_scanlines == 0)	/* strip beginning of list if possible */
	{
		ael_list = edge->next_edge;

		free(edge);
		edge = ael_list;
		(*ael_cnt)--;
		if (edge == NULL)
			return;
	}
	prev = edge;
	edge = prev->next_edge;  /* there can't be only 1 active edge */
	if (edge == NULL)
		return;

	/* now walk through the list */
	do
	{
		if (edge->num_of_scanlines == 0)
		{
			prev->next_edge = edge->next_edge;

			free(edge);
			edge = prev->next_edge;
			(*ael_cnt)--;
		}
		else
		{
			edge = edge->next_edge;
			prev = prev->next_edge;
		}
	} while (edge != NULL);
}



/*-----------------------------------------------------------------------------
-----------------------------------------------------------------------------*/
static void add_to_bucket(bucket, point1, point2)
	BUCKET **bucket;
	TmPoint32 point1;	/* passing structs... */
	TmPoint32 point2;
{
	register BUCKET *curr, *tmpBkt;
	register EDGE *edge;
	int dx, dy, y_val;

	if (point1.y == point2.y)
		return;	  /* horizontal line, will get done as a scanline */

	edge = (EDGE *)calloc(1,sizeof(EDGE));

	/* assign some data */
	dy = point2.y - point1.y;
	dx = point2.x - point1.x;
	edge->num_of_scanlines = abs(dy);

	if (!dx) /* vertical */
		edge->x_increment = 0.0;
	else
		edge->x_increment = (double) dx/dy;

	/* min_scany and max_scany will end up being the y extremes */
	/* of the polygon.	Then we need only draw between these. */
	if (point2.y < point1.y)
	{
		y_val = edge->starting_y = point2.y;
		edge->x = (double)point2.x;
		min_scany = top(min_scany,point2.y);
		max_scany = bottom(max_scany,point1.y);
	}
	else
	{
		y_val = edge->starting_y = point1.y;
		edge->x = (double)point1.x;
		min_scany = top(min_scany,point1.y);
		max_scany = bottom(max_scany,point2.y);
	}

	/* assign parameters to bucket */
	if ((*bucket)->num_of_edges == 0)  /* first bucket only */
	{
		(*bucket)->num_of_edges = 1;
		(*bucket)->scanline_y = y_val;
		(*bucket)->edge_list = edge;
		return;
	}

	/* we already have a list of buckets,		  */
	/* so start at the top to see if we:		  */
	/*	 1) need a new bucket at the top		  */
	/*	 2) need a new bucket in the middle		  */
	/*	 3) need a new bucket at the end		  */
	/*	 4) already have that y coord			  */
	/*		and just have to add the edge to it.  */

	curr = *bucket;
	if (y_val < curr->scanline_y)
	{
		/* our new y value is the smallest so far so we must */
		/* create a new bucket in front */
		if( ((*bucket) = (BUCKET *)malloc(sizeof(BUCKET))) == NULL)
		{
			fprintf(stderr,"Memory allocation error, exiting ...\n");
		}
		(*bucket)->next_buck = curr;
		(*bucket)->edge_list = edge;
		(*bucket)->num_of_edges = 1;
		(*bucket)->scanline_y = y_val;
		return;
	}
	
	/* maybe our new value == the first y value */
	if (curr->scanline_y == y_val)
	{
		edge->next_edge = curr->edge_list;
		curr->edge_list = edge;
		curr->num_of_edges++;
		return;
	}

	/* if we made it here we must walk the list */
	while ((curr->next_buck != NULL) && (curr->next_buck->scanline_y < y_val) )
		curr = curr->next_buck;

	/* If we made it to the bottom of the list, add new bucket */
	if( curr->next_buck == NULL)
	{
		if( (tmpBkt = (BUCKET *)malloc(sizeof(BUCKET))) == NULL)
		{
			fprintf(stderr,"Memory allocation error, exiting ...\n");
		}
		tmpBkt->next_buck = curr->next_buck;
		curr->next_buck = tmpBkt;
		tmpBkt->edge_list = edge;
		tmpBkt->num_of_edges = 1;
		tmpBkt->scanline_y = y_val;
		return;
	}

	/* or maybe we found a match, so just tack the edge on */
	if (curr->next_buck->scanline_y == y_val)
	{
		edge->next_edge = curr->next_buck->edge_list;
		curr->next_buck->edge_list = edge;
		curr->next_buck->num_of_edges++;
		return;
	}

	if (curr->next_buck->scanline_y > y_val) /* else insert here in the list */
	{
		if( (tmpBkt = (BUCKET *)malloc(sizeof(BUCKET))) == NULL)
		{
			fprintf(stderr,"Memory allocation error, exiting ...\n");
		}
		tmpBkt->next_buck = curr->next_buck;
		curr->next_buck = tmpBkt;
		tmpBkt->edge_list = edge;
		tmpBkt->num_of_edges = 1;
		tmpBkt->scanline_y = y_val;
		return;
	}

	/* that should cover every case! */
}



/*-----------------------------------------------------------------------------
-----------------------------------------------------------------------------*/
static void bsort()
{
	register int switched;
	register EDGE	*tmpEdge, *edge, *prev;
	
	if ((ael_list == NULL) || (ael_list->next_edge == NULL))
		return;

	do
	{
		switched = 0;
		prev = edge = ael_list;
		while (edge && edge->next_edge)
		{
			if (edge->x > edge->next_edge->x)
			{
				prev->next_edge = tmpEdge = edge->next_edge;
				edge->next_edge = tmpEdge->next_edge;
				tmpEdge->next_edge = edge;
				if (prev->next_edge != tmpEdge)
				{
					/* then the head of the list got switched, so reset */
					/* ael_list to point to the new head. */
					ael_list = prev = tmpEdge;
				}
				prev = edge;
				edge = edge->next_edge;
				switched = 1;
			}
			else
			{
				prev = edge;
				edge = edge->next_edge;
			}
		}
	} while (switched);
}

/*-----------------------------------------------------------------------------
-----------------------------------------------------------------------------*/
void pfRasterLineBytewiseCopier(Ximg_srcp, Ximg_destp, fromx, fromy, tox, toy)
	 XImage *Ximg_srcp;
	 XImage *Ximg_destp;
	 int fromx;
	 int fromy;					/* fromy should be == toy */
	 int tox;
	 int toy;
{
	char *src;
	char *dest;
	int num_of_bytes;
	int start_byte_offset;

	num_of_bytes = tox - fromx;
	if ((num_of_bytes == 0) ||
		(fromx >= Ximg_srcp->width))
		return;

	start_byte_offset = (Ximg_srcp->bytes_per_line * fromy) + fromx;

	src =  &(Ximg_srcp->data[start_byte_offset]);
	dest = &(Ximg_destp->data[start_byte_offset]);

	memcpy(dest, src, num_of_bytes);
}

/*-----------------------------------------------------------------------------
-----------------------------------------------------------------------------*/
void pfRasterLineBitwiseCopier(Ximg_srcp, Ximg_destp, fromx, fromy, tox, toy)
	 XImage *Ximg_srcp;
	 XImage *Ximg_destp;
	 int fromx;
	 int fromy;					/* fromy should be == toy */
	 int tox;
	 int toy;
{
	char *src;
	char *dest;
	int number_of_bytes;
	int start_byte_offset;

	number_of_bytes = (tox - fromx +7) / 8;
	if ((number_of_bytes == 0) ||
		(fromx >= Ximg_srcp->width))
		return;

	start_byte_offset = (Ximg_srcp->bytes_per_line * fromy) + (fromx/8);

	src =  &(Ximg_srcp->data[start_byte_offset]);
	dest = &(Ximg_destp->data[start_byte_offset]);

	memcpy(dest, src, number_of_bytes);
}


/*-----------------------------------------------------------------------------
	Builds a mask into the XImage given as the second parameter.


	7654321076543210                      v
	+-------+-------+-------+-------+-------+-------
	000000000011111111111111111111111111111000000000
	+-------+-------+-------+-------+-------+-------
	    0     ^ 1       2       3       4 ^     5
	         10                           38

	start_byte: 1
	end_byte:   4
	number_of_whole_bytes: end_byte - start_byte - 1
-----------------------------------------------------------------------------*/

void pfRasterLineMasker(Ximg_srcp, Ximg_destp, fromx, fromy, tox, toy)
	 XImage *Ximg_srcp;			/* ignored */
	 XImage *Ximg_destp;		/* OUT: an XImage 1 bit deep */
	 int fromx;					/* IN:  start of span */
	 int fromy;					/* fromy should be == toy */
	 int tox;					/* IN:  end of span */
	 int toy;
{
	register int number_of_bits;
	register int bit;
	int start_byte, end_byte;
	int start_bit, end_bit;

	number_of_bits = tox - fromx + 1; /* inclusive */
	start_bit = 7 - (fromx % 8);
	end_bit =   7 - (tox % 8);
	start_byte = (Ximg_destp->bytes_per_line * fromy) + (fromx / 8);
	end_byte =   (Ximg_destp->bytes_per_line * fromy) + (tox / 8);

	if ((number_of_bits <= 0) ||
		(fromx >= Ximg_destp->width))
		return;

	if (start_byte == end_byte)
	{
		for (bit=start_bit; bit >= end_bit; bit--)
		{
			Ximg_destp->data[start_byte] |= (unsigned char)(1 << bit);
			number_of_bits--;
		}
		return;
	}
								/* first frag */
	for (bit=start_bit; (bit >= 0) && (number_of_bits > 0); bit--)
	{
		Ximg_destp->data[start_byte] |= (unsigned char)(1 << bit);
		number_of_bits--;
	}
	if (number_of_bits == 0)
		return;
								/* full bytes */
	start_byte++;
	while (number_of_bits >= 8)
	{
		Ximg_destp->data[start_byte] |= (unsigned char)0xFF;
		start_byte++;
		number_of_bits = number_of_bits - 8;
	}
								/* last fraction */
	for (bit=7; (bit >= end_bit) && (number_of_bits > 0); bit--)
	{
		Ximg_destp->data[end_byte] |= (unsigned char)(1 << bit);
		number_of_bits--;
	}
}

