/* Copyright (C) 1983-1999 Applix, Inc.
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.
   
   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
   02111-1307, USA.
   
   Applix Inc. is the original author of this material.  Please send
   generally useful enhancements to Applix in the form of all modified 
   source files including an explanation of the changes.  For more
   information see our website at www.applixware.org.
   
   Do not remove or modify the license information above or the following
   Applix source control information from this file.
   
   Applix filename: arename.c
   Applix ASC version: 6
   Date: Mon Nov 16 13:57:58 1998
*/
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <utloc.h>

#if AXMS     /*  {  */
#ifdef Arename
#undef Arename
#endif

/*
.BEGIN MANPAGE
.NAME Arename - Change the name of a file.
.ARGUMENT src
	The address of a string which identifies the original file name.
.ARGUMENT dst
	The address of a string which identifies the new file name.
.DESCRIPTION
	Arename() provides a generic interface to change the name of a file
	or directory.  If a file or directory with the new name already exists
	and is of the same type as the original name (i.e., both directories or
	both files), then the file or directory with the new name will be removed.
.SEE ALSO
	Amove(3A)
.NOTES
	Arename() can rename directories across disk partitions.
.RETURN VALUE
	Arename() returns 0 when successful and -1 on error.
.END MANPAGE
*/

/* BEGIN SYNOPSIS */
Arename(src,dst)
char *src;
char *dst;
/* END SYNOPSIS */
{
	struct stat file_info;

	/*
	 * First we try the quick and simple approach:
	 * assume src exists, dst does not and if src is a directory
	 * we're effectively super user and src and file are in the
	 * same file partition.
	 */
#if AXMS
    if ( rename( src, dst ) == 0 )
       /* all is well */
       return( 0 );
#else
	if ( !link(src,dst) && !unlink(src) )
		return(0);
#endif

	/*
	 * So much for quick and simple.  Let's take things one step
	 * at a time.  First, do we have a source location?
	 */
	if(Astat(src,&file_info))
		return(-1);

	/*
	 * If the src is a directory AND the dst exists, then
	 * the dst had better be a directory and we should remove it.
	 * If the src is not a directory AND the dst exists, then
	 * the dst had better not be a directory and we should remove it.
	 */
	if(file_info.st_mode&S_IFDIR) {
        /* source is a directory */
		if(!Astat(dst,&file_info)) {
			if(!(file_info.st_mode&S_IFDIR)) {
                /* destination exists, and is not a directory */
				errno = ENOTDIR;
				return(-1);
			}
            /* remove the destination directory */
			if(Armdir(dst,1))
				return(-1);
		}
	}
	else if(!Astat(dst,&file_info)) {
        /* source is not a directory, and target exists */
		if(file_info.st_mode&S_IFDIR) {
            /* destination is a directory */
			errno = ENOTDIR;
			return(-1);
		}
		if(unlink(dst))
			return(-1);
	}

	/*
	 * Now we can try to link src and dst.  If src is a directory and
	 * we're not super user or src and dst are on different file systems,
	 * then we'll have to copy the src to dst.
	 */
#if AXMS
    if ( rename( src, dst ) != 0 )
        {
        if ( errno == EXDEV )
           {
           /* trying to move across devices, do a copy */
           if ( AcopyTree(src,dst) == 0 )
               /* delete the source */
               Adelete( src );
           else
               return( -1 );
           }
        return(-1);
        }
#else
	if(link(src,dst) &&
		((errno != EPERM && errno != EXDEV) || AcopyTree(src,dst)))
		return(-1);

	/*
	 * Now we can try to unlink src.  If src is a directory and
	 * we're not super user, then we'll have to remove the directory.
	 */
	if(unlink(src) && (errno != EPERM || Armdir(src,1)))
		return(-1);
#endif

	return(0);
}
#endif      /* }  end of if AXMS  */
