/* Copyright (C) 1983-1999 Applix, Inc.
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.
   
   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
   02111-1307, USA.
   
   Applix Inc. is the original author of this material.  Please send
   generally useful enhancements to Applix in the form of all modified 
   source files including an explanation of the changes.  For more
   information see our website at www.applixware.org.
   
   Do not remove or modify the license information above or the following
   Applix source control information from this file.
   
   Applix filename: image.c
   Applix ASC version: 6
   Date: Mon Nov 16 13:57:59 1998
*/
#include <stdio.h>
#include <utloc.h>
#include <afileimg.h>

#if THIMBLE
#include <adocio.h>
#include <fsrts.h>

#define IFREAD(p,sz,n,f) DocSegRead(f,p,(sz)*(n),-1)
#define IFWRITE(p,sz,n,f) DocSegWrite(f,p,(sz)*(n),-1)
#define IGETC(f) FSgetc(f)
#define IFCLOSE(f) CloseFileObject(f)
#define IFSEEK(f,p,m) DocSegSeek(f,p,m)
#define IFPUTC(c,f) FSputc(c,f)
#define IFOPEN(s,m) OpenFileObjFmode(DocP,s,m)
FILE *OpenFileObjFmode();
#else
#define IFREAD(p,sz,n,f) fread(p,sz,n,f)
#define IFWRITE(p,sz,n,f) fwrite(p,sz,n,f)
#define IGETC(f) getc(f)
#define IFCLOSE(f) fclose(f)
#define IFSEEK(f,p,m) fseek(f,p,m)
#define IFPUTC(c,f) putc(c,f)
#define IFOPEN(s,m) fopen(s,m)
#endif

/*
 * Open an image file
 */

/*
.BEGIN MANPAGE
.NAME openImage - open an ALIS image file.
.SYNOPSIS
#include <afileimg.h>
.ARGUMENT fname
	The address of a string that contains the name of the file to be opened.
.ARGUMENT mode
	The address of a string that contains the mode which the file should
	be opened with.  See fopen().
.ARGUMENT info
	The address of an imageHeader structure:
.in +5
.nf
		typedef struct {
.in +5
			int compression;
			int width;
			int height;
			int depth;
.in -5
		} imageInfo;
.fi
.in -5
			
	For image file writing, the imageHeader structure should be initialized to
	appropriate values.  For image file reading, the imageHeader structure
	will be filled in by Openi() to the values stored in the file.

	Compression can be either COMP_RAW or COMP_RUN for the 2 types
	of image compression currently supported.
.DESCRIPTION
	Openi() opens an ALIS image file for reading or writing.
	This function prepares the file for reading or writing with or
	without compression in a fashion that is transparent to the caller.
.SEE ALSO
	closei(3A) fopen(3S) scanr(3A) scanw(3A)
.RETURN VALUE
	openImage() will a pointer to an IFILE if successful and will return 0 on error.
.END MANPAGE
*/
/* BEGIN SYNOPSIS */
#if THIMBLE
IFILE *openImage(fname, mode, info,DocP)
DocInfoType *DocP;
#else
IFILE *openi(fname, mode, info)
#endif
char *fname;
char *mode;
imageInfo *info;
/* END SYNOPSIS */
{
	FILE *fp;
	IFILE *ifp;
	imageHeader hdr;

	fp = (FILE *) IFOPEN(fname, mode);
	if (fp == NULL)
		return NULL;

	ifp = (IFILE *) malloc(sizeof(IFILE));
	if (ifp == NULL)
	{
		IFCLOSE(fp);
		return NULL;
	}

	if (*mode == 'w')
	{
		ifp->mode = IM_WRITE;
		hdr.version = VERSION_1;
		hdr.compression = info->compression;
		SET_VALUE(hdr.offdata, sizeof(imageHeader));
		SET_VALUE(hdr.width, info->width);
		SET_VALUE(hdr.height, info->height);
		hdr.depth = info->depth;
		hdr.maptype = CMAP_NONE;
		SET_VALUE(hdr.maplength, 0);
		hdr.filler[0] =
			hdr.filler[1] =
				hdr.filler[2] =
					hdr.filler[3] = 0;
		IFWRITE(&hdr, 1, sizeof(imageHeader), fp);
	}
	else
	{
		ifp->mode = IM_READ;
		IFREAD(&hdr, 1, sizeof(imageHeader), fp);
		if (hdr.version != VERSION_1)
		{
			closei(ifp);
			return NULL;
		}
		info->compression =  hdr.compression;
		info->width = GET_VALUE(hdr.width);
		info->height = GET_VALUE(hdr.height);
		info->depth = hdr.depth;
		IFSEEK(fp, (long) GET_VALUE(hdr.offdata), 0);
	}
	ifp->filep = fp;
	ifp->compression = info->compression;
	switch ( info->depth )
	{
		default:
		case  1: ifp->byte_width = BYTE_WIDTH(info->width);     break;
		case  8: ifp->byte_width = info->width + info->width%2; break;
		case 32: ifp->byte_width = info->width*4;               break;
	}

	return ifp;
}

/***********************************************************************/
/*
 * Close an image file
 */

/*
.BEGIN MANPAGE
.NAME closei - Close an ALIS image file.
.SYNOPSIS
#include <afileimg.h>
.ARGUMENT ifp
	The address of an IFILE structure as returned by openImage().
.DESCRIPTION
	Closei() closes the ALIS image file previously opened by openImage().
.SEE ALSO
	openImage(3A) scanr(3A) scanw(3A)
.RETURN VALUE
	Closei() will return 0 if successful and -1 on error.
.END MANPAGE
*/

/* BEGIN SYNOPSIS */
closei(ifp)
IFILE *ifp;
/* END SYNOPSIS */
{
	FILE *fp;

	fp = ifp->filep;
	free(ifp);
	return IFCLOSE(fp);
}

/***********************************************************************/
