/* Copyright (C) 1983-1999 Applix, Inc.
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.
   
   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
   02111-1307, USA.
   
   Applix Inc. is the original author of this material.  Please send
   generally useful enhancements to Applix in the form of all modified 
   source files including an explanation of the changes.  For more
   information see our website at www.applixware.org.
   
   Do not remove or modify the license information above or the following
   Applix source control information from this file.
   
   Applix filename: parsmon.c
   Applix ASC version: 16
   Date: Thu Apr 22 13:49:16 1999
*/
#include <aos.h>

#if SYS&B4_2
#include <sys/time.h>
#else
#include <time.h>
#endif

#include <datetime.h>
#include <dt.h>

extern DUTYPE datelex();

/*
.BEGIN MANPAGE
.NAME ParseMonth - Parse a month-year string.
.SYNOPSIS
#include <datetime.h>
.ARGUMENT pdoption
	PMNULLOK parse a month and year string.
	If the input string is null or empty,
	the date-time value obtained is DTNULL.

	PMNULLCURR parse a month and year string.
	If the input string is null or empty,
	the date-time value obtained is for current system month and year.

	PMCURR obtain the current system month and year in Applix format.
	The input string is ignored.

	PMSYS given a UNIX date-time value, obtain the month and year ins
	Applix format.
	The input string is ignored.
.ARGUMENT stringP
	The address of the month-year string to be parsed.
	Ignord if pdoption is PMCURR or PMSYS.
.ARGUMENT dtvalueP
	The address to receive the Applix format date-time value.
	If pdoption is PMSYS, dtvalueP is also the address of the
	UNIX date-time value.
.ARGUMENT datefmtP
	The address to receive one of the following format codes to indicate
	the format of the string that was parsed: DTDFSLYR, DTDFABBYR, DTDFSP4YR,
	or DTDFNONE	 (if no regonizable format was found or no date-time value
	was returned.)	[This feature is not as fully implemented as it could be.
	In particular the following codes are never returned: DTDFLETYR, DTDFABB2YR 
	DTDFSP2YR.]
.DESCRIPTION
	ParseMonth() will parse a string which contains a month-year string
	and obtain the Applix format date-time value.
.SEE ALSO
	Formatdt(3A) ParseDate(3A)
.RETURN VALUE
	If ParseMonth() was successful 0 is returned,
	otherwise a non-zero value is returned and dtvalue will receive DTNULL.
.END MANPAGE
*/

/* BEGIN SYNOPSIS */
typeErr ParseMonth(pdoption,stringP,dtvalueP,datefmtP)
int pdoption;
char *stringP;
unsigned long *dtvalueP;
short *datefmtP;
/* END SYNOPSIS */
{
	/*
	 * The state table has 4 states and 4 input tokens from the lexor.  The
	 * 8 input tokens are:	<number> <month value (such as JAN)>, . or /,
	 * and <eos (end of string)>
	 *
	 * Each cell in the state table contains either a rule number or an error
	 * number and, if a rule is given, a next state number.   Since the
	 * information is bit packed in short ints, the following defines make
	 * initialization of the table easier.	The format of each entry is
	 * illustrated below:
	 *
	 * --------------------------
	 * | rrrrrrrrrrr | ssss | f |
	 * --------------------------	  where	 
	 * rrrrrrrrrrr is the rule or error number	(11 bits)
	 * ssss		   is the next state number		( 4 bits)
	 * f	 	   is the rule or error flag	( 1 bit)  1 = error, 0=
	 *			   apply rule and change to state in sss. 
	 */

#define ST 2
#define RE 32 

#define S0	(ST*0)				/*start state and terminate state */
#define S1	(ST*1)
#define S2	(ST*2)
#define S3	(ST*3)

#define E13 ((RE*13)+1)

#define OK	(RE*0)				/* rule meaning "do nothing"
								   (just do state change) */
#define R1	(RE*1)
#define R2	(RE*2)
#define R3	(RE*3)
#define R4	(RE*4)
#define R5	(RE*5)
#define R6	(RE*6)
#define R7	(RE*7)

#define NUMSTATES 4
#define NUMINPUTS 4
#define MAXSTATES 16			/* max number of states.  Must be a power of
								   2 based on the number of bits available to
								   hold state info */

	static unsigned short statetbl[NUMSTATES*NUMINPUTS] = {
		R1+S1,					/* token of 0 means "found a number"*/
		R1+S2,					/* token of 1 means
								   "found a month abbreviation such as jan"*/
		E13,					/* token of 2 means "found a . or /	  */
		R2+S0,					/* token of 3 means
								   "found <end of input string> (eos)"*/

		R3+S3,					/* STATE 1: */
		R4+S3,
		R5+S1,
		OK+S0,

		R7+S3,					/* STATE 2: */
		E13,
		R5+S2,
		OK+S0,

		E13,					/* STATE 3: */
		E13,
		E13,
		OK+S0
	};

	int sep;					/* Separator (. or /) found flag. */
	register int yr,mo,curstate;
	register int token;				/* 0=number, 1=month, 2=. or /, 4=<eos>*/
	register int tokvalue;
	long tokvaluL;
	unsigned short cell;
	char *ptr;
	long sysclock;
	struct tm *systimeP;
	typeErr retvalue;

	sep = 0;
	yr = mo = -1;
	ptr = stringP;
	retvalue = AOK;
	*datefmtP= DTDFNONE;

	if( pdoption >= PMCURR )	/* then no string to parse. */
		goto end_parse;

	curstate = 0;
	tokvaluL = (long)-99;		/* a kludge which allows datelex() to error
								   rather than ignore on leading commas. */
	while(1) {
		token = PMdatelex( &tokvaluL, &ptr );
		tokvalue = (tokvaluL < (long)32000) ? tokvaluL : 32000;
		if( *datefmtP == DTDFNONE ) {
			if( token == 2 )	/* then . or /	*/
				*datefmtP = DTDFSLYR;
			else {
				if( token == 1 ) /* then month as letters*/
					if( curstate == 0 )
						*datefmtP = DTDFABBYR;
					else
						*datefmtP = DTDFSP4YR;
			}
		}
		if ( token == -1)  {	/* unrecognizable stuff in the input stream */
			retvalue = 15;
			break;
		}
		cell = statetbl[token + (curstate*NUMINPUTS)];
		if( (cell%2) == 1)	{	/* then error! */
			retvalue = cell/RE;
			break;
		}

		switch( cell/RE ) {
		  case 1:
			mo = tokvalue;
			break;
		  case 2:
			if( pdoption == PMNULLOK ) {
				*dtvalueP = DTNULL;
				return(AOK);
			}
			break;	
		  case 3:
			if( (mo > 0) && (mo <= 12) ) {
				yr = tokvalue;
				break;
			}
			/* Fall into case 4 below. */
		  case 4:
			yr = mo;
			mo = tokvalue;
			break;
		  case 5:
			if( sep ) 
				retvalue = 18;
			sep = 1;
			break;
		  case 7:
			yr = tokvalue;
		  default:
			;
		}
		if( curstate >= 0 ) {
			curstate = cell/2;
			curstate %= MAXSTATES;
		}
		else
			curstate = -curstate;
		if ( curstate == 0 )
			break;
		if ( retvalue != AOK )
			break;
	}
	if (retvalue != AOK) {
		*dtvalueP = DTNULL;
		return( retvalue+ERRMISC );
	}

	/* fix up the date as necessary. */

end_parse:

	if( pdoption == PMSYS )
		sysclock = *dtvalueP;
	else 
		sysclock = time( (long *)0 );
	systimeP = localtime( &sysclock );

	if( yr < 0 ) {
		yr = systimeP->tm_year;
	}
	if( yr < 100 )
		yr += 1900;
	if( yr < DTTIMEBASEYEAR ) 
		retvalue = 18;			/* year is earlier than 1850 */
	if ( yr > (DTTIMEBASEYEAR + DTSPAN) ) 
		retvalue = 19;			/* year greater than 2086 */
	if( mo < 0) 
		mo = systimeP->tm_mon + 1; 
	if( (mo == 0) || (mo > 12) ) 
		retvalue = 18;

procexit:

	if (retvalue != AOK ) {
		*dtvalueP = DTNULL;
		return(	 retvalue + ERRMISC );
	}

#define FL		60/DTTIMEGRANULARITY
#define FL60		FL*60
#define FL1440		FL*1440		/* 1440 = 60*24 */
#define FL46080		FL*46080	/* 46080 = 60*24*32 */
#define FL599040	FL*599040	/* 599040 = 60*24*32*13 */
	*dtvalueP = FL1440	+		/* day value implicitly 1 */
		((unsigned long)mo	* FL46080) + /* 46080 = 60*24*32 */
		(((unsigned long)yr-DTTIMEBASEYEAR) * FL599040 );  /* 60*24*32*13 */
	return( retvalue );
}
/*-----------------------------	 PMdatelex ----------------------------*/
int PMdatelex( tokvalP, ptr )
long *tokvalP;
char **ptr;
{
	DUTYPE tok;

	tok = datelex( tokvalP, ptr );

	if( tok == DU_SLASH || tok == DU_DASH )
		return(2);
	if( tok==DU_ENDOFSTRING )
		return(3);
	if( (tok>=DU_FWKDY) && (tok<=DU_TH) || (tok>=DU_PERIOD && tok<=DU_COMMA) )
		return(-1);
	return(tok);
}

/*-----------------------------	 id_dt_code_char ----------------------------*/

static char	*hdr_ftr_code[] = {
	"#n",		/*	XLT2("Hdr/ftr codes", "#n") HF_DOCNAME */
	"#f",		/*	XLT2("Hdr/ftr codes", "#f") HF_PATHNAME */
	"#p",		/*	XLT2("Hdr/ftr codes", "#p") HF_PAGENUM */
	"#P",		/*	XLT2("Hdr/ftr codes", "#P") HF_SEQ_PAGENUM */
	"#c",		/*	XLT2("Hdr/ftr codes", "#c") HF_PAGECOUNT */
	"#v",		/*	XLT2("Hdr/ftr codes", "#v") HF_VIEWNAME */
	"#s",		/*	XLT2("Hdr/ftr codes", "#s") HF_SHEETNAME */
	"#l",		/*	XLT2("Hdr/ftr codes", "#l") HF_LASTMOD */
	"#d",		/*	XLT2("Hdr/ftr codes", "#d") HF_DATE */
	"#t",		/*	XLT2("Hdr/ftr codes", "#t") HF_TIME */
	"##",		/*	XLT2("Hdr/ftr codes", "##") HDR_CODE_CHAR */
};


int
id_dt_code_char(old_str, fmt_str)
char *old_str, *fmt_str;
{
	int	i, j, hf_len, old_len;

	old_len = strlen(old_str);

	for (i = HF_DOCNAME; i <= HF_TIME; i++)
	{
		hf_len = strlen(hdr_ftr_code[i]);

		if ( strncmp ( old_str, XLT2("Hdr/ftr codes",hdr_ftr_code[i]), hf_len ) == 0 )
		{
			if (i >= HF_LASTMOD)
			{
				if ( old_len > hf_len )
				{
					j = hf_len;
					while ( isdigit(old_str[j]) && j<old_len )
					{
						fmt_str[j - hf_len] = old_str[j];
						j++;
					}
					fmt_str[j - hf_len] = (char)0;
				}
				else
					fmt_str[0] = (char)0;
			}

			return (i);
		}
	}

	if ( strncmp ( old_str, hdr_ftr_code[i], strlen (hdr_ftr_code[i]) ) == 0 )
	{
		return (HDR_CODE_CHAR);
	}

	return (NO_HF_CODE);
}

