/* Copyright (C) 1983-1999 Applix, Inc.
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.
   
   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
   02111-1307, USA.
   
   Applix Inc. is the original author of this material.  Please send
   generally useful enhancements to Applix in the form of all modified 
   source files including an explanation of the changes.  For more
   information see our website at www.applixware.org.
   
   Do not remove or modify the license information above or the following
   Applix source control information from this file.
   
   Applix filename: scanr.c
   Applix ASC version: 3
   Date: Mon Nov 16 13:57:59 1998
*/
#include <stdio.h>
#include <aos.h>
#include <afileimg.h>

int scanr_error;

#if THIMBLE
#define IFREAD(p,sz,n,f) DocSegRead(f,p,(sz)*(n),-1)
#define IFWRITE(p,sz,n,f) DocSegWrite(f,p,(sz)*(n),-1)
#define IGETC(f) FSgetc(f)
#define IFCLOSE(f) CloseFileObject(f)
#else
#define IFREAD(p,sz,n,f) fread(p,sz,n,f)
#define IFWRITE(p,sz,n,f) fwrite(p,sz,n,f)
#define IGETC(f) getc(f)
#define IFCLOSE(f) fclose(f)
#endif

/***********************************************************************/
/*
 * Read a scanline from an image file, unpacking if nec.
 */
/*
.BEGIN MANPAGE
.NAME scanr - Read a single scanline of image data from an ALIS image file.
.SYNOPSIS
#include <afileimg.h>
.ARGUMENT ifp
	The address of an IFILE structure as returned by openi().
.ARGUMENT buf
	The address of a byte array to receive the read in image scanline.
	The array needs to be at least big enough to hold the data.
	A macro is available in <afileimg.h> to compute the required
	size for `buf':

.nf
.ti +10
BYTE_WIDTH(info->width)
.fi

	where info is the imageInfo structure as filled in by openi().
.DESCRIPTION
	Scanr() reads ALIS image file scanlines.  Each call of Scanr()
	retrieves an array of bytes containing the pixels contained 
	in the next horizontal scanline of the open image file.

	The bytes in the scanline are organized in traditional fashion,
	from left to right, where each byte contains its leftmost pixel
	in the most significant bit and the least significant bit holds
	the rightmost pixel.

	Scanr() automatically handles any decompression required of the image.
.SEE ALSO
	openi(3A) closei(3A) scanw(3A)
.RETURN VALUE
	Scanr() will return -1 if the end-of-file is encountered,
	or if the file is corrupted.  Otherwise, the actual number of
	decompressed bytes read from the image file is returned.
.END MANPAGE
*/
/* BEGIN SYNOPSIS */
scanr(ifp, buf)
IFILE *ifp;
char *buf;
/* END SYNOPSIS */
{
	if (ifp->mode != IM_READ)
		return 0;

	switch (ifp->compression) {

	case COMP_RAW:
		return IFREAD(buf, 1, ifp->byte_width, ifp->filep);

	case COMP_RUN:
		return unpacki(ifp->filep, buf, ifp->byte_width);

	default:
		return 0;
	}
}

/***********************************************************************/
/*
 * Unpack a scanline of `dnum' bytes from a file.
 */
int unpacki(fp, dst, dnum)
FILE *fp;		/* file to read bytes from */
register char *dst;	/* address of destination pointer */
int dnum;		/* number of decompressed bytes to read */
{
	register int n, c;
	int dn;

	dn = dnum;
	while (0 < dnum) {
		if ((n = IGETC(fp)) == EOF)
			return EOF;

		n &= 0xFF;

		if (n & 0x80) {
			n = 257 - n;
			if ((dnum -= n) < 0 || (c = IGETC(fp)) == EOF) {
				scanr_error = 1;
				return EOF;
			}
			while (n--)
				*dst++ = c;
		}
		else {
			n = n + 1;
			if ((dnum -= n) < 0 || n != IFREAD(dst, 1, n, fp)) {
				scanr_error = 2;
				return EOF;
			}
			dst += n;
		}
	}

	return dn;
}

