/* Copyright (C) 1983-1999 Applix, Inc.
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.
   
   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
   02111-1307, USA.
   
   Applix Inc. is the original author of this material.  Please send
   generally useful enhancements to Applix in the form of all modified 
   source files including an explanation of the changes.  For more
   information see our website at www.applixware.org.
   
   Do not remove or modify the license information above or the following
   Applix source control information from this file.
   
   Applix filename: scanw.c
   Applix ASC version: 3
   Date: Mon Nov 16 13:57:59 1998
*/
#include <stdio.h>
#include <aos.h>
#include <afileimg.h>

#if THIMBLE
#define IFREAD(p,sz,n,f) DocSegRead(f,p,(sz)*(n),-1)
#define IFWRITE(p,sz,n,f) DocSegWrite(f,p,(sz)*(n),-1)
#define IGETC(f) FSgetc(f)
#define IFCLOSE(f) CloseFileObject(f)
#define IFSEEK(f,p,m) SegSegSeek(f,p,m)
#define IFPUTC(c,f) FSfputc(c,f)
#else
#define IFREAD(p,sz,n,f) fread(p,sz,n,f)
#define IFWRITE(p,sz,n,f) fwrite(p,sz,n,f)
#define IGETC(f) getc(f)
#define IFCLOSE(f) fclose(f)
#define IFSEEK(f,p,m) fseek(f,p,m)
#define IFPUTC(c,f) putc(c,f)
#endif

#define RUN	0
#define DMP	1

#define MINRUN	3
#define MAXRUN	128
#define MAXDMP	128

int scanw_error;

/***********************************************************************/
/*
 * Write a scanline to an image file, packing if nec.
 */
/*
.BEGIN MANPAGE
.NAME scanw - Write a single scanline of image data to an ALIS image file.
.SYNOPSIS
#include <afileimg.h>
.ARGUMENT ifp
	The address of an IFILE structure as returned by openi().
.ARGUMENT buf
	The address of a byte array containing the scanline data.  It is assumed
	o contain at least 1 scanline's worth of data of a size computed
	via the "afileimg.h" macro:

.nf
.ti +10
BYTE_WIDTH(info->width)
.fi

	where info is the imageInfo structure as filled in by openi().
.DESCRIPTION
	Scanw() writes ALIS image file scanlines.  Each call of Scanw()
	copies an array of bytes containing the pixels
	representing the next horizontal scanline to the open image file.

	The bytes to be written should be organized in traditional fashion,
	from left to right, where each byte contains its leftmost pixel
	in the most significant bit and the least significant bit holds
	the rightmost pixel.

	Scanw() automatically handles any compression required of the image.
.SEE ALSO
	openi(3A) closei(3A) scanr(3A)
.RETURN VALUE
	Scanw() will return -1 if a write error is encountered.
	Otherwise, the actual number of uncompressed bytes written to the
	image file is returned.
.END MANPAGE
*/
/* BEGIN SYNOPSIS */
scanw(ifp, buf)
IFILE *ifp;
char *buf;
/* END SYNOPSIS */
{
	if (ifp->mode != IM_WRITE)
		return 0;

	switch (ifp->compression) {

	case COMP_RAW:
		return IFWRITE(buf, 1, ifp->byte_width, ifp->filep);

	case COMP_RUN:
		return packi(ifp->filep, buf, ifp->byte_width);

	default:
		return 0;
	}
}

/***********************************************************************/
/*
 * Pack a scanline of `snum' bytes into a file.
 */
int packi(fp, src, snum)
FILE *fp;		/* where to write the bytes */
register char *src;	/* address of sources pointer */
int snum;		/* count of source bytes to pack */
{
	register char *buf;
	register int nbuf, rstart;
	int i, mode;
	char buffer[260];
	char c, lastc;
	
	buf = buffer + 1;
	buf[0] = lastc = c = *src++;
	nbuf = 1;
	rstart = 0;

	mode = DMP;

	for (i = 1; i < snum; i++) {

		buf[nbuf++] = c = *src++;

		switch (mode) {

		case DMP:
			if (MAXDMP < nbuf) {
				buffer[0] = nbuf - 2;
				if (IFWRITE(buffer, 1, nbuf, fp) == EOF) {
					scanw_error = 1;
					return EOF;
				}
				buf[0] = c;
				nbuf = 1;
				rstart = 0;
				break;
			}

			if (c != lastc) {
				rstart = nbuf - 1;	/* 1st of poss run */
				break;
			}

			if (rstart && nbuf - rstart < MINRUN)
				break;

			if (0 < rstart) {
				buffer[0] = rstart - 1;
				if (IFWRITE(buffer, 1, rstart + 1, fp) == EOF) {
					scanw_error = 2;
					return EOF;
				}
			}

			mode = RUN;
			break;

		case RUN:
			if (c == lastc && nbuf - rstart <= MAXRUN)
				break;

			IFPUTC(rstart - nbuf + 2, fp);
			IFPUTC(lastc, fp);
#if !THIMBLE
			if (ferror(fp) != 0) {	
				scanw_error = 3;
				return EOF;
			}
#endif
			buf[0] = c;
			nbuf = 1;
			rstart = 0;

			mode = DMP;
			break;
		}

		lastc = c;
	}

	if (mode == DMP) {
		buffer[0] = nbuf - 1;
		if (IFWRITE(buffer, 1, nbuf + 1, fp) == EOF) {
			scanw_error = 4;
			return EOF;
		}
	}
	else {
		IFPUTC(rstart - nbuf + 1, fp);
		IFPUTC(lastc, fp);
#if !THIMBLE
		if (ferror(fp) != 0) {	
			scanw_error = 5;
			return EOF;
		}
#endif
	}

	return snum;
}

